# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

ANTS_REGISTRATION_METADATA = Metadata(
    id="c1ca233b97fb2c34e284eb1f804669cd3da9255a.boutiques",
    name="antsRegistration",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


AntsRegistrationInitialMovingTransformParameters = typing.TypedDict('AntsRegistrationInitialMovingTransformParameters', {
    "__STYX_TYPE__": typing.Literal["initial_moving_transform"],
    "initial_moving_transform": InputPathType,
})


AntsRegistrationInitialMovingTransformUseInverseParameters = typing.TypedDict('AntsRegistrationInitialMovingTransformUseInverseParameters', {
    "__STYX_TYPE__": typing.Literal["initial_moving_transform_use_inverse"],
    "initial_moving_transform": InputPathType,
    "use_inverse": typing.NotRequired[typing.Literal[0, 1] | None],
})


AntsRegistrationInitialMovingTransformInitializationFeatureParameters = typing.TypedDict('AntsRegistrationInitialMovingTransformInitializationFeatureParameters', {
    "__STYX_TYPE__": typing.Literal["initial_moving_transform_initialization_feature"],
    "fixed_image": InputPathType,
    "moving_image": InputPathType,
    "initialization_feature": typing.Literal[0, 1, 2],
})


AntsRegistrationTransformRigidParameters = typing.TypedDict('AntsRegistrationTransformRigidParameters', {
    "__STYX_TYPE__": typing.Literal["transform_rigid"],
    "gradient_step": float,
})


AntsRegistrationTransformAffineParameters = typing.TypedDict('AntsRegistrationTransformAffineParameters', {
    "__STYX_TYPE__": typing.Literal["transform_affine"],
    "gradient_step": float,
})


AntsRegistrationTransformCompositeAffineParameters = typing.TypedDict('AntsRegistrationTransformCompositeAffineParameters', {
    "__STYX_TYPE__": typing.Literal["transform_composite_affine"],
    "gradient_step": float,
})


AntsRegistrationTransformSimilarityParameters = typing.TypedDict('AntsRegistrationTransformSimilarityParameters', {
    "__STYX_TYPE__": typing.Literal["transform_similarity"],
    "gradient_step": float,
})


AntsRegistrationTransformTranslationParameters = typing.TypedDict('AntsRegistrationTransformTranslationParameters', {
    "__STYX_TYPE__": typing.Literal["transform_translation"],
    "gradient_step": float,
})


AntsRegistrationTransformBsplineParameters = typing.TypedDict('AntsRegistrationTransformBsplineParameters', {
    "__STYX_TYPE__": typing.Literal["transform_bspline"],
    "gradient_step": float,
    "mesh_size_at_base_level": float,
})


AntsRegistrationTransformGaussianDisplacementFieldParameters = typing.TypedDict('AntsRegistrationTransformGaussianDisplacementFieldParameters', {
    "__STYX_TYPE__": typing.Literal["transform_gaussian_displacement_field"],
    "gradient_step": float,
    "update_field_variance_in_voxel_space": float,
    "total_field_variance_in_voxel_space": float,
})


AntsRegistrationTransformBsplineDisplacementFieldParameters = typing.TypedDict('AntsRegistrationTransformBsplineDisplacementFieldParameters', {
    "__STYX_TYPE__": typing.Literal["transform_bspline_displacement_field"],
    "gradient_step": float,
    "update_field_mesh_size_at_base_level": float,
    "total_field_mesh_size_at_base_level": typing.NotRequired[float | None],
    "spline_order": typing.NotRequired[float | None],
})


AntsRegistrationTransformTimeVaryingVelocityFieldParameters = typing.TypedDict('AntsRegistrationTransformTimeVaryingVelocityFieldParameters', {
    "__STYX_TYPE__": typing.Literal["transform_time_varying_velocity_field"],
    "gradient_step": float,
    "number_of_time_indices": float,
    "update_field_variance_in_voxel_space": float,
    "update_field_time_variance": float,
    "total_field_variance_in_voxel_space": float,
    "total_field_time_variance": float,
})


AntsRegistrationTransformTimeVaryingBsplineVelocityFieldParameters = typing.TypedDict('AntsRegistrationTransformTimeVaryingBsplineVelocityFieldParameters', {
    "__STYX_TYPE__": typing.Literal["transform_time_varying_bspline_velocity_field"],
    "gradient_step": float,
    "velocity_field_mesh_size": float,
    "number_of_time_point_samples": typing.NotRequired[float | None],
    "spline_order": typing.NotRequired[float | None],
})


AntsRegistrationTransformSynParameters = typing.TypedDict('AntsRegistrationTransformSynParameters', {
    "__STYX_TYPE__": typing.Literal["transform_syn"],
    "gradient_step": float,
    "update_field_variance_in_voxel_space": float,
    "total_field_variance_in_voxel_space": float,
})


AntsRegistrationTransformBsplineSynParameters = typing.TypedDict('AntsRegistrationTransformBsplineSynParameters', {
    "__STYX_TYPE__": typing.Literal["transform_bspline_syn"],
    "gradient_step": float,
    "update_field_mesh_size_at_base_level": float,
    "total_field_mesh_size_at_base_level": typing.NotRequired[float | None],
    "spline_order": typing.NotRequired[float | None],
})


AntsRegistrationTransformExponentialParameters = typing.TypedDict('AntsRegistrationTransformExponentialParameters', {
    "__STYX_TYPE__": typing.Literal["transform_exponential"],
    "gradient_step": float,
    "update_field_variance_in_voxel_space": float,
    "velocity_field_variance_in_voxel_space": float,
    "number_of_integration_steps": float,
})


AntsRegistrationTransformBsplineExponentialParameters = typing.TypedDict('AntsRegistrationTransformBsplineExponentialParameters', {
    "__STYX_TYPE__": typing.Literal["transform_bspline_exponential"],
    "gradient_step": float,
    "update_field_mesh_size_at_base_level": float,
    "velocity_field_mesh_size_at_base_level": typing.NotRequired[float | None],
    "number_of_integration_steps": typing.NotRequired[float | None],
    "spline_order": typing.NotRequired[float | None],
})


AntsRegistrationMetricAntsNeighbourhoodCrossCorrelationParameters = typing.TypedDict('AntsRegistrationMetricAntsNeighbourhoodCrossCorrelationParameters', {
    "__STYX_TYPE__": typing.Literal["metric_ants_neighbourhood_cross_correlation"],
    "fixed_image": str,
    "moving_image": str,
    "metric_weight": float,
    "radius": typing.NotRequired[float | None],
    "sampling_strategy": typing.NotRequired[typing.Literal["None", "Regular", "Random"] | None],
    "sampling_percentage": typing.NotRequired[float | None],
    "use_gradient_filter": typing.NotRequired[typing.Literal["true", "false"] | None],
})


AntsRegistrationMetricMutualInformationParameters = typing.TypedDict('AntsRegistrationMetricMutualInformationParameters', {
    "__STYX_TYPE__": typing.Literal["metric_mutual_information"],
    "fixed_image": str,
    "moving_image": str,
    "metric_weight": float,
    "number_of_bins": typing.NotRequired[float | None],
    "sampling_strategy": typing.NotRequired[typing.Literal["None", "Regular", "Random"] | None],
    "sampling_percentage": typing.NotRequired[float | None],
    "use_gradient_filter": typing.NotRequired[typing.Literal["true", "false"] | None],
})


AntsRegistrationMetricMattesParameters = typing.TypedDict('AntsRegistrationMetricMattesParameters', {
    "__STYX_TYPE__": typing.Literal["metric_mattes"],
    "fixed_image": str,
    "moving_image": str,
    "metric_weight": float,
    "number_of_bins": typing.NotRequired[float | None],
    "sampling_strategy": typing.NotRequired[typing.Literal["None", "Regular", "Random"] | None],
    "sampling_percentage": typing.NotRequired[float | None],
    "use_gradient_filter": typing.NotRequired[typing.Literal["true", "false"] | None],
})


AntsRegistrationMetricMeanSquaresParameters = typing.TypedDict('AntsRegistrationMetricMeanSquaresParameters', {
    "__STYX_TYPE__": typing.Literal["metric_mean_squares"],
    "fixed_image": str,
    "moving_image": str,
    "metric_weight": float,
    "radius": typing.NotRequired[float | None],
    "sampling_strategy": typing.NotRequired[typing.Literal["None", "Regular", "Random"] | None],
    "sampling_percentage": typing.NotRequired[float | None],
    "use_gradient_filter": typing.NotRequired[typing.Literal["true", "false"] | None],
})


AntsRegistrationMetricDemonsParameters = typing.TypedDict('AntsRegistrationMetricDemonsParameters', {
    "__STYX_TYPE__": typing.Literal["metric_demons"],
    "fixed_image": str,
    "moving_image": str,
    "metric_weight": float,
    "number_of_bins": typing.NotRequired[float | None],
    "sampling_strategy": typing.NotRequired[typing.Literal["None", "Regular", "Random"] | None],
    "sampling_percentage": typing.NotRequired[float | None],
    "use_gradient_filter": typing.NotRequired[typing.Literal["true", "false"] | None],
})


AntsRegistrationMetricGlobalCorrelationParameters = typing.TypedDict('AntsRegistrationMetricGlobalCorrelationParameters', {
    "__STYX_TYPE__": typing.Literal["metric_global_correlation"],
    "fixed_image": str,
    "moving_image": str,
    "metric_weight": float,
    "radius": typing.NotRequired[float | None],
    "sampling_strategy": typing.NotRequired[typing.Literal["None", "Regular", "Random"] | None],
    "sampling_percentage": typing.NotRequired[float | None],
    "use_gradient_filter": typing.NotRequired[typing.Literal["true", "false"] | None],
})


AntsRegistrationMetricEuclideanIcpParameters = typing.TypedDict('AntsRegistrationMetricEuclideanIcpParameters', {
    "__STYX_TYPE__": typing.Literal["metric_euclidean_icp"],
    "fixed_point_set": str,
    "moving_point_set": str,
    "metric_weight": float,
    "sampling_percentage": typing.NotRequired[float | None],
    "boundary_points_only": typing.NotRequired[typing.Literal["0"] | None],
})


AntsRegistrationMetricPointSetExpectationParameters = typing.TypedDict('AntsRegistrationMetricPointSetExpectationParameters', {
    "__STYX_TYPE__": typing.Literal["metric_point_set_expectation"],
    "metric_weight": float,
    "point_set_sigma": typing.NotRequired[float | None],
    "sampling_percentage": typing.NotRequired[float | None],
})


AntsRegistrationMetricJensenHavrdaCharvetTsallisParameters = typing.TypedDict('AntsRegistrationMetricJensenHavrdaCharvetTsallisParameters', {
    "__STYX_TYPE__": typing.Literal["metric_jensen_havrda_charvet_tsallis"],
    "fixed_point_set": str,
    "moving_point_set": str,
    "metric_weight": float,
    "sampling_percentage": typing.NotRequired[float | None],
    "boundary_points_only": typing.NotRequired[typing.Literal["0"] | None],
    "point_set_sigma": typing.NotRequired[float | None],
    "k_neighborhood": typing.NotRequired[float | None],
})


AntsRegistrationMetricIgdmParameters = typing.TypedDict('AntsRegistrationMetricIgdmParameters', {
    "__STYX_TYPE__": typing.Literal["metric_igdm"],
})


AntsRegistrationConvergenceParameters = typing.TypedDict('AntsRegistrationConvergenceParameters', {
    "__STYX_TYPE__": typing.Literal["convergence"],
    "convergence": str,
    "convergence_threshold": float,
    "convergence_window_size": int,
})


AntsRegistrationStageParameters = typing.TypedDict('AntsRegistrationStageParameters', {
    "__STYX_TYPE__": typing.Literal["stage"],
    "transform": typing.Union[AntsRegistrationTransformRigidParameters, AntsRegistrationTransformAffineParameters, AntsRegistrationTransformCompositeAffineParameters, AntsRegistrationTransformSimilarityParameters, AntsRegistrationTransformTranslationParameters, AntsRegistrationTransformBsplineParameters, AntsRegistrationTransformGaussianDisplacementFieldParameters, AntsRegistrationTransformBsplineDisplacementFieldParameters, AntsRegistrationTransformTimeVaryingVelocityFieldParameters, AntsRegistrationTransformTimeVaryingBsplineVelocityFieldParameters, AntsRegistrationTransformSynParameters, AntsRegistrationTransformBsplineSynParameters, AntsRegistrationTransformExponentialParameters, AntsRegistrationTransformBsplineExponentialParameters],
    "metric": typing.Union[AntsRegistrationMetricAntsNeighbourhoodCrossCorrelationParameters, AntsRegistrationMetricMutualInformationParameters, AntsRegistrationMetricMattesParameters, AntsRegistrationMetricMeanSquaresParameters, AntsRegistrationMetricDemonsParameters, AntsRegistrationMetricGlobalCorrelationParameters, AntsRegistrationMetricEuclideanIcpParameters, AntsRegistrationMetricPointSetExpectationParameters, AntsRegistrationMetricJensenHavrdaCharvetTsallisParameters, AntsRegistrationMetricIgdmParameters],
    "convergence": AntsRegistrationConvergenceParameters,
    "smoothing_sigmas": str,
    "shrink_factors": str,
    "use_histogram_matching": typing.NotRequired[typing.Literal[0, 1] | None],
})


AntsRegistrationWinsorizeImageIntensitiesParameters = typing.TypedDict('AntsRegistrationWinsorizeImageIntensitiesParameters', {
    "__STYX_TYPE__": typing.Literal["winsorize_image_intensities"],
    "lower_quantile": float,
    "upper_quantile": float,
})


AntsRegistrationMasksParameters = typing.TypedDict('AntsRegistrationMasksParameters', {
    "__STYX_TYPE__": typing.Literal["masks"],
    "fixed_mask": typing.NotRequired[str | None],
    "moving_mask": typing.NotRequired[str | None],
})


AntsRegistrationParameters = typing.TypedDict('AntsRegistrationParameters', {
    "__STYX_TYPE__": typing.Literal["antsRegistration"],
    "dimensionality": typing.NotRequired[typing.Literal[2, 3, 4] | None],
    "output": typing.NotRequired[str | None],
    "save_state": typing.NotRequired[str | None],
    "restore_state": typing.NotRequired[str | None],
    "write_composite_transform": typing.NotRequired[typing.Literal[0, 1] | None],
    "print_similarity_measure_interval": typing.NotRequired[int | None],
    "write_interval_volumes": typing.NotRequired[int | None],
    "collapse_output_transforms": typing.NotRequired[typing.Literal[1, 0] | None],
    "initialize_transforms_per_stage": typing.NotRequired[typing.Literal[1, 0] | None],
    "interpolation": typing.NotRequired[typing.Literal["Linear", "NearestNeighbor", "MultiLabel", "Gaussian", "BSpline", "CosineWindowedSinc", "WelchWindowedSinc", "HammingWindowedSinc", "LanczosWindowedSinc", "GenericLabel"] | None],
    "restrict_deformation": typing.NotRequired[list[typing.Literal[0, 1]] | None],
    "initial_fixed_transform": typing.NotRequired[str | None],
    "initial_moving_transform": typing.NotRequired[typing.Union[AntsRegistrationInitialMovingTransformParameters, AntsRegistrationInitialMovingTransformUseInverseParameters, AntsRegistrationInitialMovingTransformInitializationFeatureParameters] | None],
    "stages": list[AntsRegistrationStageParameters],
    "winsorize_image_intensities": typing.NotRequired[AntsRegistrationWinsorizeImageIntensitiesParameters | None],
    "masks": typing.NotRequired[AntsRegistrationMasksParameters | None],
    "minc": typing.NotRequired[typing.Literal[0, 1] | None],
    "random_seed": typing.NotRequired[int | None],
    "verbose": typing.NotRequired[typing.Literal[0, 1] | None],
    "float": typing.NotRequired[typing.Literal[0, 1] | None],
})


def dyn_cargs(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "antsRegistration": ants_registration_cargs,
        "initial_moving_transform": ants_registration_initial_moving_transform_cargs,
        "initial_moving_transform_use_inverse": ants_registration_initial_moving_transform_use_inverse_cargs,
        "initial_moving_transform_initialization_feature": ants_registration_initial_moving_transform_initialization_feature_cargs,
        "stage": ants_registration_stage_cargs,
        "transform_rigid": ants_registration_transform_rigid_cargs,
        "transform_affine": ants_registration_transform_affine_cargs,
        "transform_composite_affine": ants_registration_transform_composite_affine_cargs,
        "transform_similarity": ants_registration_transform_similarity_cargs,
        "transform_translation": ants_registration_transform_translation_cargs,
        "transform_bspline": ants_registration_transform_bspline_cargs,
        "transform_gaussian_displacement_field": ants_registration_transform_gaussian_displacement_field_cargs,
        "transform_bspline_displacement_field": ants_registration_transform_bspline_displacement_field_cargs,
        "transform_time_varying_velocity_field": ants_registration_transform_time_varying_velocity_field_cargs,
        "transform_time_varying_bspline_velocity_field": ants_registration_transform_time_varying_bspline_velocity_field_cargs,
        "transform_syn": ants_registration_transform_syn_cargs,
        "transform_bspline_syn": ants_registration_transform_bspline_syn_cargs,
        "transform_exponential": ants_registration_transform_exponential_cargs,
        "transform_bspline_exponential": ants_registration_transform_bspline_exponential_cargs,
        "metric_ants_neighbourhood_cross_correlation": ants_registration_metric_ants_neighbourhood_cross_correlation_cargs,
        "metric_mutual_information": ants_registration_metric_mutual_information_cargs,
        "metric_mattes": ants_registration_metric_mattes_cargs,
        "metric_mean_squares": ants_registration_metric_mean_squares_cargs,
        "metric_demons": ants_registration_metric_demons_cargs,
        "metric_global_correlation": ants_registration_metric_global_correlation_cargs,
        "metric_euclidean_icp": ants_registration_metric_euclidean_icp_cargs,
        "metric_point_set_expectation": ants_registration_metric_point_set_expectation_cargs,
        "metric_jensen_havrda_charvet_tsallis": ants_registration_metric_jensen_havrda_charvet_tsallis_cargs,
        "metric_igdm": ants_registration_metric_igdm_cargs,
        "convergence": ants_registration_convergence_cargs,
        "winsorize_image_intensities": ants_registration_winsorize_image_intensities_cargs,
        "masks": ants_registration_masks_cargs,
    }.get(t)


def dyn_outputs(
    t: str,
) -> typing.Any:
    """
    Get build outputs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build outputs function.
    """
    return {
        "antsRegistration": ants_registration_outputs,
    }.get(t)


def ants_registration_initial_moving_transform_params(
    initial_moving_transform: InputPathType,
) -> AntsRegistrationInitialMovingTransformParameters:
    """
    Build parameters.
    
    Args:
        initial_moving_transform:.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "initial_moving_transform",
        "initial_moving_transform": initial_moving_transform,
    }
    return params


def ants_registration_initial_moving_transform_cargs(
    params: AntsRegistrationInitialMovingTransformParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(execution.input_file(params.get("initial_moving_transform")))
    return cargs


def ants_registration_initial_moving_transform_use_inverse_params(
    initial_moving_transform: InputPathType,
    use_inverse: typing.Literal[0, 1] | None = None,
) -> AntsRegistrationInitialMovingTransformUseInverseParameters:
    """
    Build parameters.
    
    Args:
        initial_moving_transform:.
        use_inverse: Use the inverse of the initial moving transform.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "initial_moving_transform_use_inverse",
        "initial_moving_transform": initial_moving_transform,
    }
    if use_inverse is not None:
        params["use_inverse"] = use_inverse
    return params


def ants_registration_initial_moving_transform_use_inverse_cargs(
    params: AntsRegistrationInitialMovingTransformUseInverseParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    if params.get("use_inverse") is not None:
        cargs.append("[" + execution.input_file(params.get("initial_moving_transform")) + "," + str(params.get("use_inverse")) + "]")
    return cargs


def ants_registration_initial_moving_transform_initialization_feature_params(
    fixed_image: InputPathType,
    moving_image: InputPathType,
    initialization_feature: typing.Literal[0, 1, 2],
) -> AntsRegistrationInitialMovingTransformInitializationFeatureParameters:
    """
    Build parameters.
    
    Args:
        fixed_image:.
        moving_image:.
        initialization_feature: Initialization feature. 0: Geometric center of\
            images, 1: Image intensities, 2: Origin of images.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "initial_moving_transform_initialization_feature",
        "fixed_image": fixed_image,
        "moving_image": moving_image,
        "initialization_feature": initialization_feature,
    }
    return params


def ants_registration_initial_moving_transform_initialization_feature_cargs(
    params: AntsRegistrationInitialMovingTransformInitializationFeatureParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("[" + execution.input_file(params.get("fixed_image")) + "," + execution.input_file(params.get("moving_image")) + "," + str(params.get("initialization_feature")) + "]")
    return cargs


def ants_registration_transform_rigid_params(
    gradient_step: float,
) -> AntsRegistrationTransformRigidParameters:
    """
    Build parameters.
    
    Args:
        gradient_step:.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "transform_rigid",
        "gradient_step": gradient_step,
    }
    return params


def ants_registration_transform_rigid_cargs(
    params: AntsRegistrationTransformRigidParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("Rigid[" + str(params.get("gradient_step")) + "]")
    return cargs


def ants_registration_transform_affine_params(
    gradient_step: float,
) -> AntsRegistrationTransformAffineParameters:
    """
    Build parameters.
    
    Args:
        gradient_step:.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "transform_affine",
        "gradient_step": gradient_step,
    }
    return params


def ants_registration_transform_affine_cargs(
    params: AntsRegistrationTransformAffineParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("Affine[" + str(params.get("gradient_step")) + "]")
    return cargs


def ants_registration_transform_composite_affine_params(
    gradient_step: float,
) -> AntsRegistrationTransformCompositeAffineParameters:
    """
    Build parameters.
    
    Args:
        gradient_step:.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "transform_composite_affine",
        "gradient_step": gradient_step,
    }
    return params


def ants_registration_transform_composite_affine_cargs(
    params: AntsRegistrationTransformCompositeAffineParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("CompositeAffine[" + str(params.get("gradient_step")) + "]")
    return cargs


def ants_registration_transform_similarity_params(
    gradient_step: float,
) -> AntsRegistrationTransformSimilarityParameters:
    """
    Build parameters.
    
    Args:
        gradient_step:.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "transform_similarity",
        "gradient_step": gradient_step,
    }
    return params


def ants_registration_transform_similarity_cargs(
    params: AntsRegistrationTransformSimilarityParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("Similarity[" + str(params.get("gradient_step")) + "]")
    return cargs


def ants_registration_transform_translation_params(
    gradient_step: float,
) -> AntsRegistrationTransformTranslationParameters:
    """
    Build parameters.
    
    Args:
        gradient_step:.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "transform_translation",
        "gradient_step": gradient_step,
    }
    return params


def ants_registration_transform_translation_cargs(
    params: AntsRegistrationTransformTranslationParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("Translation[" + str(params.get("gradient_step")) + "]")
    return cargs


def ants_registration_transform_bspline_params(
    gradient_step: float,
    mesh_size_at_base_level: float,
) -> AntsRegistrationTransformBsplineParameters:
    """
    Build parameters.
    
    Args:
        gradient_step:.
        mesh_size_at_base_level:.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "transform_bspline",
        "gradient_step": gradient_step,
        "mesh_size_at_base_level": mesh_size_at_base_level,
    }
    return params


def ants_registration_transform_bspline_cargs(
    params: AntsRegistrationTransformBsplineParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("BSpline[" + str(params.get("gradient_step")) + "," + str(params.get("mesh_size_at_base_level")) + "]")
    return cargs


def ants_registration_transform_gaussian_displacement_field_params(
    gradient_step: float,
    update_field_variance_in_voxel_space: float,
    total_field_variance_in_voxel_space: float,
) -> AntsRegistrationTransformGaussianDisplacementFieldParameters:
    """
    Build parameters.
    
    Args:
        gradient_step:.
        update_field_variance_in_voxel_space:.
        total_field_variance_in_voxel_space:.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "transform_gaussian_displacement_field",
        "gradient_step": gradient_step,
        "update_field_variance_in_voxel_space": update_field_variance_in_voxel_space,
        "total_field_variance_in_voxel_space": total_field_variance_in_voxel_space,
    }
    return params


def ants_registration_transform_gaussian_displacement_field_cargs(
    params: AntsRegistrationTransformGaussianDisplacementFieldParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("GaussianDisplacementField[" + str(params.get("gradient_step")) + "," + str(params.get("update_field_variance_in_voxel_space")) + "," + str(params.get("total_field_variance_in_voxel_space")) + "]")
    return cargs


def ants_registration_transform_bspline_displacement_field_params(
    gradient_step: float,
    update_field_mesh_size_at_base_level: float,
    total_field_mesh_size_at_base_level: float | None = None,
    spline_order: float | None = None,
) -> AntsRegistrationTransformBsplineDisplacementFieldParameters:
    """
    Build parameters.
    
    Args:
        gradient_step:.
        update_field_mesh_size_at_base_level:.
        total_field_mesh_size_at_base_level:.
        spline_order:.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "transform_bspline_displacement_field",
        "gradient_step": gradient_step,
        "update_field_mesh_size_at_base_level": update_field_mesh_size_at_base_level,
    }
    if total_field_mesh_size_at_base_level is not None:
        params["total_field_mesh_size_at_base_level"] = total_field_mesh_size_at_base_level
    if spline_order is not None:
        params["spline_order"] = spline_order
    return params


def ants_registration_transform_bspline_displacement_field_cargs(
    params: AntsRegistrationTransformBsplineDisplacementFieldParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    if params.get("total_field_mesh_size_at_base_level") is not None or params.get("spline_order") is not None:
        cargs.append("BSplineDisplacementField[" + str(params.get("gradient_step")) + "," + str(params.get("update_field_mesh_size_at_base_level")) + "," + (str(params.get("total_field_mesh_size_at_base_level")) if (params.get("total_field_mesh_size_at_base_level") is not None) else "") + (str(params.get("spline_order")) if (params.get("spline_order") is not None) else "") + "]")
    return cargs


def ants_registration_transform_time_varying_velocity_field_params(
    gradient_step: float,
    number_of_time_indices: float,
    update_field_variance_in_voxel_space: float,
    update_field_time_variance: float,
    total_field_variance_in_voxel_space: float,
    total_field_time_variance: float,
) -> AntsRegistrationTransformTimeVaryingVelocityFieldParameters:
    """
    Build parameters.
    
    Args:
        gradient_step:.
        number_of_time_indices:.
        update_field_variance_in_voxel_space:.
        update_field_time_variance:.
        total_field_variance_in_voxel_space:.
        total_field_time_variance:.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "transform_time_varying_velocity_field",
        "gradient_step": gradient_step,
        "number_of_time_indices": number_of_time_indices,
        "update_field_variance_in_voxel_space": update_field_variance_in_voxel_space,
        "update_field_time_variance": update_field_time_variance,
        "total_field_variance_in_voxel_space": total_field_variance_in_voxel_space,
        "total_field_time_variance": total_field_time_variance,
    }
    return params


def ants_registration_transform_time_varying_velocity_field_cargs(
    params: AntsRegistrationTransformTimeVaryingVelocityFieldParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("TimeVaryingVelocityField[" + str(params.get("gradient_step")) + "," + str(params.get("number_of_time_indices")) + "," + str(params.get("update_field_variance_in_voxel_space")) + "," + str(params.get("update_field_time_variance")) + "," + str(params.get("total_field_variance_in_voxel_space")) + "," + str(params.get("total_field_time_variance")) + "]")
    return cargs


def ants_registration_transform_time_varying_bspline_velocity_field_params(
    gradient_step: float,
    velocity_field_mesh_size: float,
    number_of_time_point_samples: float | None = None,
    spline_order: float | None = None,
) -> AntsRegistrationTransformTimeVaryingBsplineVelocityFieldParameters:
    """
    Build parameters.
    
    Args:
        gradient_step:.
        velocity_field_mesh_size:.
        number_of_time_point_samples:.
        spline_order:.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "transform_time_varying_bspline_velocity_field",
        "gradient_step": gradient_step,
        "velocity_field_mesh_size": velocity_field_mesh_size,
    }
    if number_of_time_point_samples is not None:
        params["number_of_time_point_samples"] = number_of_time_point_samples
    if spline_order is not None:
        params["spline_order"] = spline_order
    return params


def ants_registration_transform_time_varying_bspline_velocity_field_cargs(
    params: AntsRegistrationTransformTimeVaryingBsplineVelocityFieldParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    if params.get("number_of_time_point_samples") is not None or params.get("spline_order") is not None:
        cargs.append("TimeVaryingBSplineVelocityField[" + str(params.get("gradient_step")) + "," + str(params.get("velocity_field_mesh_size")) + "," + (str(params.get("number_of_time_point_samples")) if (params.get("number_of_time_point_samples") is not None) else "") + "," + (str(params.get("spline_order")) if (params.get("spline_order") is not None) else "") + "]")
    return cargs


def ants_registration_transform_syn_params(
    gradient_step: float,
    update_field_variance_in_voxel_space: float,
    total_field_variance_in_voxel_space: float,
) -> AntsRegistrationTransformSynParameters:
    """
    Build parameters.
    
    Args:
        gradient_step:.
        update_field_variance_in_voxel_space:.
        total_field_variance_in_voxel_space:.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "transform_syn",
        "gradient_step": gradient_step,
        "update_field_variance_in_voxel_space": update_field_variance_in_voxel_space,
        "total_field_variance_in_voxel_space": total_field_variance_in_voxel_space,
    }
    return params


def ants_registration_transform_syn_cargs(
    params: AntsRegistrationTransformSynParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("SyN[" + str(params.get("gradient_step")) + "," + str(params.get("update_field_variance_in_voxel_space")) + "," + str(params.get("total_field_variance_in_voxel_space")) + "]")
    return cargs


def ants_registration_transform_bspline_syn_params(
    gradient_step: float,
    update_field_mesh_size_at_base_level: float,
    total_field_mesh_size_at_base_level: float | None = None,
    spline_order: float | None = None,
) -> AntsRegistrationTransformBsplineSynParameters:
    """
    Build parameters.
    
    Args:
        gradient_step:.
        update_field_mesh_size_at_base_level:.
        total_field_mesh_size_at_base_level:.
        spline_order:.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "transform_bspline_syn",
        "gradient_step": gradient_step,
        "update_field_mesh_size_at_base_level": update_field_mesh_size_at_base_level,
    }
    if total_field_mesh_size_at_base_level is not None:
        params["total_field_mesh_size_at_base_level"] = total_field_mesh_size_at_base_level
    if spline_order is not None:
        params["spline_order"] = spline_order
    return params


def ants_registration_transform_bspline_syn_cargs(
    params: AntsRegistrationTransformBsplineSynParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    if params.get("total_field_mesh_size_at_base_level") is not None or params.get("spline_order") is not None:
        cargs.append("BSplineSyN[" + str(params.get("gradient_step")) + "," + str(params.get("update_field_mesh_size_at_base_level")) + "," + (str(params.get("total_field_mesh_size_at_base_level")) if (params.get("total_field_mesh_size_at_base_level") is not None) else "") + "," + (str(params.get("spline_order")) if (params.get("spline_order") is not None) else "") + "]")
    return cargs


def ants_registration_transform_exponential_params(
    gradient_step: float,
    update_field_variance_in_voxel_space: float,
    velocity_field_variance_in_voxel_space: float,
    number_of_integration_steps: float,
) -> AntsRegistrationTransformExponentialParameters:
    """
    Build parameters.
    
    Args:
        gradient_step:.
        update_field_variance_in_voxel_space:.
        velocity_field_variance_in_voxel_space:.
        number_of_integration_steps:.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "transform_exponential",
        "gradient_step": gradient_step,
        "update_field_variance_in_voxel_space": update_field_variance_in_voxel_space,
        "velocity_field_variance_in_voxel_space": velocity_field_variance_in_voxel_space,
        "number_of_integration_steps": number_of_integration_steps,
    }
    return params


def ants_registration_transform_exponential_cargs(
    params: AntsRegistrationTransformExponentialParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("Exponential[" + str(params.get("gradient_step")) + "," + str(params.get("update_field_variance_in_voxel_space")) + "," + str(params.get("velocity_field_variance_in_voxel_space")) + "," + str(params.get("number_of_integration_steps")) + "]")
    return cargs


def ants_registration_transform_bspline_exponential_params(
    gradient_step: float,
    update_field_mesh_size_at_base_level: float,
    velocity_field_mesh_size_at_base_level: float | None = None,
    number_of_integration_steps: float | None = None,
    spline_order: float | None = None,
) -> AntsRegistrationTransformBsplineExponentialParameters:
    """
    Build parameters.
    
    Args:
        gradient_step:.
        update_field_mesh_size_at_base_level:.
        velocity_field_mesh_size_at_base_level:.
        number_of_integration_steps:.
        spline_order:.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "transform_bspline_exponential",
        "gradient_step": gradient_step,
        "update_field_mesh_size_at_base_level": update_field_mesh_size_at_base_level,
    }
    if velocity_field_mesh_size_at_base_level is not None:
        params["velocity_field_mesh_size_at_base_level"] = velocity_field_mesh_size_at_base_level
    if number_of_integration_steps is not None:
        params["number_of_integration_steps"] = number_of_integration_steps
    if spline_order is not None:
        params["spline_order"] = spline_order
    return params


def ants_registration_transform_bspline_exponential_cargs(
    params: AntsRegistrationTransformBsplineExponentialParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    if params.get("velocity_field_mesh_size_at_base_level") is not None or params.get("number_of_integration_steps") is not None or params.get("spline_order") is not None:
        cargs.append("BSplineExponential[" + str(params.get("gradient_step")) + "," + str(params.get("update_field_mesh_size_at_base_level")) + "," + (str(params.get("velocity_field_mesh_size_at_base_level")) if (params.get("velocity_field_mesh_size_at_base_level") is not None) else "") + "," + (str(params.get("number_of_integration_steps")) if (params.get("number_of_integration_steps") is not None) else "") + "," + (str(params.get("spline_order")) if (params.get("spline_order") is not None) else "") + "]")
    return cargs


def ants_registration_metric_ants_neighbourhood_cross_correlation_params(
    fixed_image: str,
    moving_image: str,
    metric_weight: float,
    radius: float | None = None,
    sampling_strategy: typing.Literal["None", "Regular", "Random"] | None = None,
    sampling_percentage: float | None = None,
    use_gradient_filter: typing.Literal["true", "false"] | None = None,
) -> AntsRegistrationMetricAntsNeighbourhoodCrossCorrelationParameters:
    """
    Build parameters.
    
    Args:
        fixed_image:.
        moving_image:.
        metric_weight:.
        radius:.
        sampling_strategy:.
        sampling_percentage:.
        use_gradient_filter:.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "metric_ants_neighbourhood_cross_correlation",
        "fixed_image": fixed_image,
        "moving_image": moving_image,
        "metric_weight": metric_weight,
    }
    if radius is not None:
        params["radius"] = radius
    if sampling_strategy is not None:
        params["sampling_strategy"] = sampling_strategy
    if sampling_percentage is not None:
        params["sampling_percentage"] = sampling_percentage
    if use_gradient_filter is not None:
        params["use_gradient_filter"] = use_gradient_filter
    return params


def ants_registration_metric_ants_neighbourhood_cross_correlation_cargs(
    params: AntsRegistrationMetricAntsNeighbourhoodCrossCorrelationParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    if params.get("radius") is not None or params.get("sampling_strategy") is not None or params.get("sampling_percentage") is not None or params.get("use_gradient_filter") is not None:
        cargs.append("CC[" + params.get("fixed_image") + "," + params.get("moving_image") + "," + str(params.get("metric_weight")) + "," + (str(params.get("radius")) if (params.get("radius") is not None) else "") + "," + (params.get("sampling_strategy") if (params.get("sampling_strategy") is not None) else "") + "," + (str(params.get("sampling_percentage")) if (params.get("sampling_percentage") is not None) else "") + "," + (params.get("use_gradient_filter") if (params.get("use_gradient_filter") is not None) else "") + "]")
    return cargs


def ants_registration_metric_mutual_information_params(
    fixed_image: str,
    moving_image: str,
    metric_weight: float,
    number_of_bins: float | None = None,
    sampling_strategy: typing.Literal["None", "Regular", "Random"] | None = None,
    sampling_percentage: float | None = None,
    use_gradient_filter: typing.Literal["true", "false"] | None = None,
) -> AntsRegistrationMetricMutualInformationParameters:
    """
    Build parameters.
    
    Args:
        fixed_image:.
        moving_image:.
        metric_weight:.
        number_of_bins:.
        sampling_strategy:.
        sampling_percentage:.
        use_gradient_filter:.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "metric_mutual_information",
        "fixed_image": fixed_image,
        "moving_image": moving_image,
        "metric_weight": metric_weight,
    }
    if number_of_bins is not None:
        params["number_of_bins"] = number_of_bins
    if sampling_strategy is not None:
        params["sampling_strategy"] = sampling_strategy
    if sampling_percentage is not None:
        params["sampling_percentage"] = sampling_percentage
    if use_gradient_filter is not None:
        params["use_gradient_filter"] = use_gradient_filter
    return params


def ants_registration_metric_mutual_information_cargs(
    params: AntsRegistrationMetricMutualInformationParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    if params.get("number_of_bins") is not None or params.get("sampling_strategy") is not None or params.get("sampling_percentage") is not None or params.get("use_gradient_filter") is not None:
        cargs.append("MI[" + params.get("fixed_image") + "," + params.get("moving_image") + "," + str(params.get("metric_weight")) + "," + (str(params.get("number_of_bins")) if (params.get("number_of_bins") is not None) else "") + "," + (params.get("sampling_strategy") if (params.get("sampling_strategy") is not None) else "") + "," + (str(params.get("sampling_percentage")) if (params.get("sampling_percentage") is not None) else "") + "," + (params.get("use_gradient_filter") if (params.get("use_gradient_filter") is not None) else "") + "]")
    return cargs


def ants_registration_metric_mattes_params(
    fixed_image: str,
    moving_image: str,
    metric_weight: float,
    number_of_bins: float | None = None,
    sampling_strategy: typing.Literal["None", "Regular", "Random"] | None = None,
    sampling_percentage: float | None = None,
    use_gradient_filter: typing.Literal["true", "false"] | None = None,
) -> AntsRegistrationMetricMattesParameters:
    """
    Build parameters.
    
    Args:
        fixed_image:.
        moving_image:.
        metric_weight:.
        number_of_bins:.
        sampling_strategy:.
        sampling_percentage:.
        use_gradient_filter:.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "metric_mattes",
        "fixed_image": fixed_image,
        "moving_image": moving_image,
        "metric_weight": metric_weight,
    }
    if number_of_bins is not None:
        params["number_of_bins"] = number_of_bins
    if sampling_strategy is not None:
        params["sampling_strategy"] = sampling_strategy
    if sampling_percentage is not None:
        params["sampling_percentage"] = sampling_percentage
    if use_gradient_filter is not None:
        params["use_gradient_filter"] = use_gradient_filter
    return params


def ants_registration_metric_mattes_cargs(
    params: AntsRegistrationMetricMattesParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    if params.get("number_of_bins") is not None or params.get("sampling_strategy") is not None or params.get("sampling_percentage") is not None or params.get("use_gradient_filter") is not None:
        cargs.append("Mattes[" + params.get("fixed_image") + "," + params.get("moving_image") + "," + str(params.get("metric_weight")) + "," + (str(params.get("number_of_bins")) if (params.get("number_of_bins") is not None) else "") + "," + (params.get("sampling_strategy") if (params.get("sampling_strategy") is not None) else "") + "," + (str(params.get("sampling_percentage")) if (params.get("sampling_percentage") is not None) else "") + "," + (params.get("use_gradient_filter") if (params.get("use_gradient_filter") is not None) else "") + "]")
    return cargs


def ants_registration_metric_mean_squares_params(
    fixed_image: str,
    moving_image: str,
    metric_weight: float,
    radius: float | None = None,
    sampling_strategy: typing.Literal["None", "Regular", "Random"] | None = None,
    sampling_percentage: float | None = None,
    use_gradient_filter: typing.Literal["true", "false"] | None = None,
) -> AntsRegistrationMetricMeanSquaresParameters:
    """
    Build parameters.
    
    Args:
        fixed_image:.
        moving_image:.
        metric_weight:.
        radius:.
        sampling_strategy:.
        sampling_percentage:.
        use_gradient_filter:.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "metric_mean_squares",
        "fixed_image": fixed_image,
        "moving_image": moving_image,
        "metric_weight": metric_weight,
    }
    if radius is not None:
        params["radius"] = radius
    if sampling_strategy is not None:
        params["sampling_strategy"] = sampling_strategy
    if sampling_percentage is not None:
        params["sampling_percentage"] = sampling_percentage
    if use_gradient_filter is not None:
        params["use_gradient_filter"] = use_gradient_filter
    return params


def ants_registration_metric_mean_squares_cargs(
    params: AntsRegistrationMetricMeanSquaresParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    if params.get("radius") is not None or params.get("sampling_strategy") is not None or params.get("sampling_percentage") is not None or params.get("use_gradient_filter") is not None:
        cargs.append("MeanSquares[" + params.get("fixed_image") + "," + params.get("moving_image") + "," + str(params.get("metric_weight")) + "," + (str(params.get("radius")) if (params.get("radius") is not None) else "") + "," + (params.get("sampling_strategy") if (params.get("sampling_strategy") is not None) else "") + "," + (str(params.get("sampling_percentage")) if (params.get("sampling_percentage") is not None) else "") + "," + (params.get("use_gradient_filter") if (params.get("use_gradient_filter") is not None) else "") + "]")
    return cargs


def ants_registration_metric_demons_params(
    fixed_image: str,
    moving_image: str,
    metric_weight: float,
    number_of_bins: float | None = None,
    sampling_strategy: typing.Literal["None", "Regular", "Random"] | None = None,
    sampling_percentage: float | None = None,
    use_gradient_filter: typing.Literal["true", "false"] | None = None,
) -> AntsRegistrationMetricDemonsParameters:
    """
    Build parameters.
    
    Args:
        fixed_image:.
        moving_image:.
        metric_weight:.
        number_of_bins:.
        sampling_strategy:.
        sampling_percentage:.
        use_gradient_filter:.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "metric_demons",
        "fixed_image": fixed_image,
        "moving_image": moving_image,
        "metric_weight": metric_weight,
    }
    if number_of_bins is not None:
        params["number_of_bins"] = number_of_bins
    if sampling_strategy is not None:
        params["sampling_strategy"] = sampling_strategy
    if sampling_percentage is not None:
        params["sampling_percentage"] = sampling_percentage
    if use_gradient_filter is not None:
        params["use_gradient_filter"] = use_gradient_filter
    return params


def ants_registration_metric_demons_cargs(
    params: AntsRegistrationMetricDemonsParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    if params.get("number_of_bins") is not None or params.get("sampling_strategy") is not None or params.get("sampling_percentage") is not None or params.get("use_gradient_filter") is not None:
        cargs.append("Demons[" + params.get("fixed_image") + "," + params.get("moving_image") + "," + str(params.get("metric_weight")) + "," + (str(params.get("number_of_bins")) if (params.get("number_of_bins") is not None) else "") + "," + (params.get("sampling_strategy") if (params.get("sampling_strategy") is not None) else "") + "," + (str(params.get("sampling_percentage")) if (params.get("sampling_percentage") is not None) else "") + "," + (params.get("use_gradient_filter") if (params.get("use_gradient_filter") is not None) else "") + "]")
    return cargs


def ants_registration_metric_global_correlation_params(
    fixed_image: str,
    moving_image: str,
    metric_weight: float,
    radius: float | None = None,
    sampling_strategy: typing.Literal["None", "Regular", "Random"] | None = None,
    sampling_percentage: float | None = None,
    use_gradient_filter: typing.Literal["true", "false"] | None = None,
) -> AntsRegistrationMetricGlobalCorrelationParameters:
    """
    Build parameters.
    
    Args:
        fixed_image:.
        moving_image:.
        metric_weight:.
        radius:.
        sampling_strategy:.
        sampling_percentage:.
        use_gradient_filter:.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "metric_global_correlation",
        "fixed_image": fixed_image,
        "moving_image": moving_image,
        "metric_weight": metric_weight,
    }
    if radius is not None:
        params["radius"] = radius
    if sampling_strategy is not None:
        params["sampling_strategy"] = sampling_strategy
    if sampling_percentage is not None:
        params["sampling_percentage"] = sampling_percentage
    if use_gradient_filter is not None:
        params["use_gradient_filter"] = use_gradient_filter
    return params


def ants_registration_metric_global_correlation_cargs(
    params: AntsRegistrationMetricGlobalCorrelationParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    if params.get("radius") is not None or params.get("sampling_strategy") is not None or params.get("sampling_percentage") is not None or params.get("use_gradient_filter") is not None:
        cargs.append("GC[" + params.get("fixed_image") + "," + params.get("moving_image") + "," + str(params.get("metric_weight")) + "," + (str(params.get("radius")) if (params.get("radius") is not None) else "") + "," + (params.get("sampling_strategy") if (params.get("sampling_strategy") is not None) else "") + "," + (str(params.get("sampling_percentage")) if (params.get("sampling_percentage") is not None) else "") + "," + (params.get("use_gradient_filter") if (params.get("use_gradient_filter") is not None) else "") + "]")
    return cargs


def ants_registration_metric_euclidean_icp_params(
    fixed_point_set: str,
    moving_point_set: str,
    metric_weight: float,
    sampling_percentage: float | None = None,
    boundary_points_only: typing.Literal["0"] | None = None,
) -> AntsRegistrationMetricEuclideanIcpParameters:
    """
    Build parameters.
    
    Args:
        fixed_point_set:.
        moving_point_set:.
        metric_weight:.
        sampling_percentage:.
        boundary_points_only:.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "metric_euclidean_icp",
        "fixed_point_set": fixed_point_set,
        "moving_point_set": moving_point_set,
        "metric_weight": metric_weight,
    }
    if sampling_percentage is not None:
        params["sampling_percentage"] = sampling_percentage
    if boundary_points_only is not None:
        params["boundary_points_only"] = boundary_points_only
    return params


def ants_registration_metric_euclidean_icp_cargs(
    params: AntsRegistrationMetricEuclideanIcpParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    if params.get("sampling_percentage") is not None or params.get("boundary_points_only") is not None:
        cargs.append("ICP[" + params.get("fixed_point_set") + "," + params.get("moving_point_set") + "," + str(params.get("metric_weight")) + "," + (str(params.get("sampling_percentage")) if (params.get("sampling_percentage") is not None) else "") + "," + (params.get("boundary_points_only") if (params.get("boundary_points_only") is not None) else "") + "]")
    return cargs


def ants_registration_metric_point_set_expectation_params(
    metric_weight: float,
    point_set_sigma: float | None = None,
    sampling_percentage: float | None = None,
) -> AntsRegistrationMetricPointSetExpectationParameters:
    """
    Build parameters.
    
    Args:
        metric_weight:.
        point_set_sigma:.
        sampling_percentage:.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "metric_point_set_expectation",
        "metric_weight": metric_weight,
    }
    if point_set_sigma is not None:
        params["point_set_sigma"] = point_set_sigma
    if sampling_percentage is not None:
        params["sampling_percentage"] = sampling_percentage
    return params


def ants_registration_metric_point_set_expectation_cargs(
    params: AntsRegistrationMetricPointSetExpectationParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    if params.get("point_set_sigma") is not None or params.get("sampling_percentage") is not None:
        cargs.append("PSE[[FIXED_IMAGE],[MOVING_IMAGE]," + str(params.get("metric_weight")) + ",," + (str(params.get("point_set_sigma")) if (params.get("point_set_sigma") is not None) else "") + "[SAMPLING_STRATEGY]," + (str(params.get("sampling_percentage")) if (params.get("sampling_percentage") is not None) else "") + "[USE_GRADIENT_FILTER]]")
    return cargs


def ants_registration_metric_jensen_havrda_charvet_tsallis_params(
    fixed_point_set: str,
    moving_point_set: str,
    metric_weight: float,
    sampling_percentage: float | None = None,
    boundary_points_only: typing.Literal["0"] | None = None,
    point_set_sigma: float | None = None,
    k_neighborhood: float | None = None,
) -> AntsRegistrationMetricJensenHavrdaCharvetTsallisParameters:
    """
    Build parameters.
    
    Args:
        fixed_point_set:.
        moving_point_set:.
        metric_weight:.
        sampling_percentage:.
        boundary_points_only:.
        point_set_sigma:.
        k_neighborhood:.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "metric_jensen_havrda_charvet_tsallis",
        "fixed_point_set": fixed_point_set,
        "moving_point_set": moving_point_set,
        "metric_weight": metric_weight,
    }
    if sampling_percentage is not None:
        params["sampling_percentage"] = sampling_percentage
    if boundary_points_only is not None:
        params["boundary_points_only"] = boundary_points_only
    if point_set_sigma is not None:
        params["point_set_sigma"] = point_set_sigma
    if k_neighborhood is not None:
        params["k_neighborhood"] = k_neighborhood
    return params


def ants_registration_metric_jensen_havrda_charvet_tsallis_cargs(
    params: AntsRegistrationMetricJensenHavrdaCharvetTsallisParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    if params.get("sampling_percentage") is not None or params.get("boundary_points_only") is not None or params.get("point_set_sigma") is not None or params.get("k_neighborhood") is not None:
        cargs.append("JHCT[" + params.get("fixed_point_set") + "," + params.get("moving_point_set") + "," + str(params.get("metric_weight")) + "," + (str(params.get("sampling_percentage")) if (params.get("sampling_percentage") is not None) else "") + "," + (params.get("boundary_points_only") if (params.get("boundary_points_only") is not None) else "") + "," + (str(params.get("point_set_sigma")) if (params.get("point_set_sigma") is not None) else "") + "," + (str(params.get("k_neighborhood")) if (params.get("k_neighborhood") is not None) else "") + "]")
    return cargs


def ants_registration_metric_igdm_params(
) -> AntsRegistrationMetricIgdmParameters:
    """
    Build parameters.
    
    Args:
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "metric_igdm",
    }
    return params


def ants_registration_metric_igdm_cargs(
    params: AntsRegistrationMetricIgdmParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("IGDM[")
    return cargs


def ants_registration_convergence_params(
    convergence: str,
    convergence_threshold: float,
    convergence_window_size: int,
) -> AntsRegistrationConvergenceParameters:
    """
    Build parameters.
    
    Args:
        convergence:.
        convergence_threshold:.
        convergence_window_size:.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "convergence",
        "convergence": convergence,
        "convergence_threshold": convergence_threshold,
        "convergence_window_size": convergence_window_size,
    }
    return params


def ants_registration_convergence_cargs(
    params: AntsRegistrationConvergenceParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("[" + params.get("convergence") + "," + str(params.get("convergence_threshold")) + "," + str(params.get("convergence_window_size")) + "]")
    return cargs


def ants_registration_stage_params(
    transform: typing.Union[AntsRegistrationTransformRigidParameters, AntsRegistrationTransformAffineParameters, AntsRegistrationTransformCompositeAffineParameters, AntsRegistrationTransformSimilarityParameters, AntsRegistrationTransformTranslationParameters, AntsRegistrationTransformBsplineParameters, AntsRegistrationTransformGaussianDisplacementFieldParameters, AntsRegistrationTransformBsplineDisplacementFieldParameters, AntsRegistrationTransformTimeVaryingVelocityFieldParameters, AntsRegistrationTransformTimeVaryingBsplineVelocityFieldParameters, AntsRegistrationTransformSynParameters, AntsRegistrationTransformBsplineSynParameters, AntsRegistrationTransformExponentialParameters, AntsRegistrationTransformBsplineExponentialParameters],
    metric: typing.Union[AntsRegistrationMetricAntsNeighbourhoodCrossCorrelationParameters, AntsRegistrationMetricMutualInformationParameters, AntsRegistrationMetricMattesParameters, AntsRegistrationMetricMeanSquaresParameters, AntsRegistrationMetricDemonsParameters, AntsRegistrationMetricGlobalCorrelationParameters, AntsRegistrationMetricEuclideanIcpParameters, AntsRegistrationMetricPointSetExpectationParameters, AntsRegistrationMetricJensenHavrdaCharvetTsallisParameters, AntsRegistrationMetricIgdmParameters],
    convergence: AntsRegistrationConvergenceParameters,
    smoothing_sigmas: str,
    shrink_factors: str,
    use_histogram_matching: typing.Literal[0, 1] | None = None,
) -> AntsRegistrationStageParameters:
    """
    Build parameters.
    
    Args:
        transform: Several transform options are available. The gradientStep or\
            learningRate characterizes the gradient descent optimization and is\
            scaled appropriately for each transform using the shift scales\
            estimator. Subsequent parameters are transform-specific and can be\
            determined from the usage. For the B-spline transforms one can also\
            specify the smoothing in terms of spline distance (i.e. knot spacing).
        metric: These image metrics are available--- CC: ANTS neighborhood\
            cross correlation, MI: Mutual information, Demons: (Thirion),\
            MeanSquares, and GC: Global Correlation. The "metricWeight" variable is\
            used to modulate the per stage weighting of the metrics. The metrics\
            can also employ a sampling strategy defined by a sampling percentage.\
            The sampling strategy defaults to 'None' (aka a dense sampling of one\
            sample per voxel), otherwise it defines a point set over which to\
            optimize the metric. The point set can be on a regular lattice or a\
            random lattice of points slightly perturbed to minimize aliasing\
            artifacts. samplingPercentage defines the fraction of points to select\
            from the domain. useGradientFilter specifies whether a smoothingfilter\
            is applied when estimating the metric gradient.In addition, three point\
            set metrics are available: Euclidean (ICP), Point-set expectation\
            (PSE), and Jensen-Havrda-Charvet-Tsallis (JHCT).
        convergence: Convergence is determined from the number of iterations\
            per level and is determined by fitting a line to the normalized energy\
            profile of the last N iterations (where N is specified by the window\
            size) and determining the slope which is then compared with the\
            convergence threshold.
        smoothing_sigmas: Specify the sigma of gaussian smoothing at each\
            level. Units are given in terms of voxels ('vox') or physical spacing\
            ('mm'). Example usage is '4x2x1mm' and '4x2x1vox' where no units\
            implies voxel spacing.
        shrink_factors: Specify the shrink factor for the virtual domain\
            (typically the fixed image) at each level.
        use_histogram_matching: Use histogram matching.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "stage",
        "transform": transform,
        "metric": metric,
        "convergence": convergence,
        "smoothing_sigmas": smoothing_sigmas,
        "shrink_factors": shrink_factors,
    }
    if use_histogram_matching is not None:
        params["use_histogram_matching"] = use_histogram_matching
    return params


def ants_registration_stage_cargs(
    params: AntsRegistrationStageParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "--transform",
        *dyn_cargs(params.get("transform")["__STYXTYPE__"])(params.get("transform"), execution)
    ])
    cargs.extend([
        "--metric",
        *dyn_cargs(params.get("metric")["__STYXTYPE__"])(params.get("metric"), execution)
    ])
    cargs.extend([
        "--convergence",
        *dyn_cargs(params.get("convergence")["__STYXTYPE__"])(params.get("convergence"), execution)
    ])
    cargs.extend([
        "--smoothing-sigmas",
        params.get("smoothing_sigmas")
    ])
    cargs.extend([
        "--shrink-factors",
        params.get("shrink_factors")
    ])
    if params.get("use_histogram_matching") is not None:
        cargs.extend([
            "--use-histogram-matching",
            str(params.get("use_histogram_matching"))
        ])
    return cargs


def ants_registration_winsorize_image_intensities_params(
    lower_quantile: float,
    upper_quantile: float,
) -> AntsRegistrationWinsorizeImageIntensitiesParameters:
    """
    Build parameters.
    
    Args:
        lower_quantile:.
        upper_quantile:.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "winsorize_image_intensities",
        "lower_quantile": lower_quantile,
        "upper_quantile": upper_quantile,
    }
    return params


def ants_registration_winsorize_image_intensities_cargs(
    params: AntsRegistrationWinsorizeImageIntensitiesParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("[" + str(params.get("lower_quantile")) + str(params.get("upper_quantile")) + "]")
    return cargs


def ants_registration_masks_params(
    fixed_mask: str | None = None,
    moving_mask: str | None = None,
) -> AntsRegistrationMasksParameters:
    """
    Build parameters.
    
    Args:
        fixed_mask:.
        moving_mask:.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "masks",
    }
    if fixed_mask is not None:
        params["fixed_mask"] = fixed_mask
    if moving_mask is not None:
        params["moving_mask"] = moving_mask
    return params


def ants_registration_masks_cargs(
    params: AntsRegistrationMasksParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    if params.get("fixed_mask") is not None or params.get("moving_mask") is not None:
        cargs.append("[" + (params.get("fixed_mask") if (params.get("fixed_mask") is not None) else "") + (params.get("moving_mask") if (params.get("moving_mask") is not None) else "") + "]")
    return cargs


class AntsRegistrationOutputs(typing.NamedTuple):
    """
    Output object returned when calling `ants_registration(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    generic_affine: OutputPathType | None
    """The output is the generic affine transformation matrix."""
    inverse_warped: OutputPathType | None
    """The output is the warped fixed image."""
    inverse_warp: OutputPathType | None
    """The output is the inverse warp field."""
    warped: OutputPathType | None
    """The output is the warped moving image."""
    warp: OutputPathType | None
    """The output is the warp field."""


def ants_registration_params(
    stages: list[AntsRegistrationStageParameters],
    dimensionality: typing.Literal[2, 3, 4] | None = None,
    output: str | None = None,
    save_state: str | None = None,
    restore_state: str | None = None,
    write_composite_transform: typing.Literal[0, 1] | None = None,
    print_similarity_measure_interval: int | None = None,
    write_interval_volumes: int | None = None,
    collapse_output_transforms: typing.Literal[1, 0] | None = None,
    initialize_transforms_per_stage: typing.Literal[1, 0] | None = None,
    interpolation: typing.Literal["Linear", "NearestNeighbor", "MultiLabel", "Gaussian", "BSpline", "CosineWindowedSinc", "WelchWindowedSinc", "HammingWindowedSinc", "LanczosWindowedSinc", "GenericLabel"] | None = None,
    restrict_deformation: list[typing.Literal[0, 1]] | None = None,
    initial_fixed_transform: str | None = None,
    initial_moving_transform: typing.Union[AntsRegistrationInitialMovingTransformParameters, AntsRegistrationInitialMovingTransformUseInverseParameters, AntsRegistrationInitialMovingTransformInitializationFeatureParameters] | None = None,
    winsorize_image_intensities: AntsRegistrationWinsorizeImageIntensitiesParameters | None = None,
    masks: AntsRegistrationMasksParameters | None = None,
    minc: typing.Literal[0, 1] | None = None,
    random_seed: int | None = None,
    verbose: typing.Literal[0, 1] | None = None,
    float_: typing.Literal[0, 1] | None = None,
) -> AntsRegistrationParameters:
    """
    Build parameters.
    
    Args:
        stages: Stages of the registration process.
        dimensionality: This option forces the image to be treated as a\
            specified-dimensional image. If not specified, we try to infer the\
            dimensionality from the input image.
        output: Specify the output transform prefix (output format is .nii.gz\
            ). Optionally, one can choose to warp the moving image to the fixed\
            space and, if the inverse transform exists, one can also output the\
            warped fixed image. Note that only the images specified in the first\
            metric call are warped. Use antsApplyTransforms to warp other images\
            using the resultant transform(s). When a composite transform is not\
            specified, linear transforms are specified with a '.mat' suffix and\
            displacement fields with a 'Warp.nii.gz' suffix (and\
            'InverseWarp.nii.gz', when applicable. In addition, for velocity-based\
            transforms, the full velocity field is written to file\
            ('VelocityField.nii.gz') as long as the collapse transforms flag is\
            turned off ('-z 0').
        save_state: Specify the output file for the current state of the\
            registration. The state file is written to an hdf5 composite file. It\
            is specially usefull if we want to save the current state of a SyN\
            registration to the disk, so we can load and restore that later to\
            continue the next registration process directly started from the last\
            saved state. The output file of this flag is the same as the\
            write-composite-transform, unless the last transform is a SyN\
            transform. In that case, the inverse displacement field of the SyN\
            transform is also added to the output composite transform. Again notice\
            that this file cannot be treated as a transform, and restore-state\
            option must be used to load the written file by this flag.
        restore_state: Specify the initial state of the registration which get\
            immediately used to directly initialize the registration process. The\
            flag is mutually exclusive with other intialization flags.If this flag\
            is used, none of the initial-moving-transform and\
            initial-fixed-transform cannot be used.
        write_composite_transform: Boolean specifying whether or not the\
            composite transform (and its inverse, if it exists) should be written\
            to an hdf5 composite file. This is false by default so that only the\
            transform for each stage is written to file.
        print_similarity_measure_interval: Prints out the CC similarity metric\
            measure between the full-size input fixed and the transformed moving\
            images at each iteration a value of 0 (the default) indicates that the\
            full scale computation should not take placeany value greater than 0\
            represents the interval of full scale metric computation.
        write_interval_volumes: Writes out the output volume at each iteration.\
            It helps to present the registration process as a short movie a value\
            of 0 (the default) indicates that this option should not take placeany\
            value greater than 0 represents the interval between the iterations\
            which outputs are written to the disk.
        collapse_output_transforms: Collapse output transforms. Specifically,\
            enabling this option combines all adjacent transforms where possible.\
            All adjacent linear transforms are written to disk in the form of an\
            itk affine transform (called xxxGenericAffine.mat).\
            Similarly, all adjacent displacement field transforms are combined\
            when written to disk (e.g. xxxWarp.nii.gz and xxxInverseWarp.nii.gz\
            (if available)). Also, an output composite transform including the\
            collapsed transforms is written to the disk (called\
            outputCollapsed(Inverse)Composite).
        initialize_transforms_per_stage: Initialize linear transforms from the\
            previous stage. By enabling this option, the current linear stage\
            transform is directly intialized from the previous stage's linear\
            transform; this allows multiple linear stages to be run where each\
            stage directly updates the estimated linear transform from the previous\
            stage. (e.g. Translation -> Rigid -> Affine).
        interpolation: Several interpolation options are available in ITK.\
            These have all been made available. Currently the interpolator choice\
            is only used to warp (and possibly inverse warp) the final output\
            image(s).
        restrict_deformation: This option allows the user to restrict the\
            optimization of the displacement field, translation, rigid or affine\
            transform on a per-component basis. For example, if one wants to limit\
            the deformation or rotation of 3-D volume to the first two dimensions,\
            this is possible by specifying a weight vector of '1x1x0' for a\
            deformation field or '1x1x0x1x1x0' for a rigid transformation.\
            Low-dimensional restriction only works if there are no preceding\
            transformations.All stages up to and including the desired stage must\
            have this option specified,even if they should not be restricted (in\
            which case specify 1x1x1...).
        initial_fixed_transform: Specify the initial fixed transform(s) which\
            get immediately incorporated into the composite transform. The order of\
            the transforms is stack-esque in that the last transform specified on\
            the command line is the first to be applied. In addition to\
            initialization with ITK transforms, the user can perform an initial\
            translation alignment by specifying the fixed and moving images and\
            selecting an initialization feature. These features include using the\
            geometric center of the images (=0), the image intensities (=1), or the\
            origin of the images (=2).
        initial_moving_transform: Specify the initial moving transform(s) which\
            get immediately incorporated into the composite transform. The order of\
            the transforms is stack-esque in that the last transform specified on\
            the command line is the first to be applied. In addition to\
            initialization with ITK transforms, the user can perform an initial\
            translation alignment by specifying the fixed and moving images and\
            selecting an initialization feature. These features include using the\
            geometric center of the images (=0), the image intensities (=1), or the\
            origin of the images (=2).
        winsorize_image_intensities: Winsorize data based on specified\
            quantiles.
        masks: Image masks to limit voxels considered by the metric. Two\
            options are allowed for mask specification: 1) Either the user\
            specifies a single mask to be used for all stages or 2) the user\
            specifies a mask for each stage. With the latter one can select to\
            which stages masks are applied by supplying valid file names. If the\
            file does not exist, a mask will not be used for that stage. Note that\
            we handle the fixed and moving masks separately to enforce this\
            constraint.
        minc: Use MINC file formats for transformations.
        random_seed: Random seed.
        verbose: Verbose output.
        float_: Use 'float' instead of 'double' for computations.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "antsRegistration",
        "stages": stages,
    }
    if dimensionality is not None:
        params["dimensionality"] = dimensionality
    if output is not None:
        params["output"] = output
    if save_state is not None:
        params["save_state"] = save_state
    if restore_state is not None:
        params["restore_state"] = restore_state
    if write_composite_transform is not None:
        params["write_composite_transform"] = write_composite_transform
    if print_similarity_measure_interval is not None:
        params["print_similarity_measure_interval"] = print_similarity_measure_interval
    if write_interval_volumes is not None:
        params["write_interval_volumes"] = write_interval_volumes
    if collapse_output_transforms is not None:
        params["collapse_output_transforms"] = collapse_output_transforms
    if initialize_transforms_per_stage is not None:
        params["initialize_transforms_per_stage"] = initialize_transforms_per_stage
    if interpolation is not None:
        params["interpolation"] = interpolation
    if restrict_deformation is not None:
        params["restrict_deformation"] = restrict_deformation
    if initial_fixed_transform is not None:
        params["initial_fixed_transform"] = initial_fixed_transform
    if initial_moving_transform is not None:
        params["initial_moving_transform"] = initial_moving_transform
    if winsorize_image_intensities is not None:
        params["winsorize_image_intensities"] = winsorize_image_intensities
    if masks is not None:
        params["masks"] = masks
    if minc is not None:
        params["minc"] = minc
    if random_seed is not None:
        params["random_seed"] = random_seed
    if verbose is not None:
        params["verbose"] = verbose
    if float_ is not None:
        params["float"] = float_
    return params


def ants_registration_cargs(
    params: AntsRegistrationParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("antsRegistration")
    if params.get("dimensionality") is not None:
        cargs.extend([
            "--dimensionality",
            str(params.get("dimensionality"))
        ])
    if params.get("output") is not None:
        cargs.extend([
            "-o",
            params.get("output")
        ])
    if params.get("save_state") is not None:
        cargs.extend([
            "-j",
            params.get("save_state")
        ])
    if params.get("restore_state") is not None:
        cargs.extend([
            "-k",
            params.get("restore_state")
        ])
    if params.get("write_composite_transform") is not None:
        cargs.extend([
            "-a",
            str(params.get("write_composite_transform"))
        ])
    if params.get("print_similarity_measure_interval") is not None:
        cargs.extend([
            "-p",
            str(params.get("print_similarity_measure_interval"))
        ])
    if params.get("write_interval_volumes") is not None:
        cargs.extend([
            "--write-interval-volumes",
            str(params.get("write_interval_volumes"))
        ])
    if params.get("collapse_output_transforms") is not None:
        cargs.extend([
            "--collapse-output-transforms",
            str(params.get("collapse_output_transforms"))
        ])
    if params.get("initialize_transforms_per_stage") is not None:
        cargs.extend([
            "-i",
            str(params.get("initialize_transforms_per_stage"))
        ])
    if params.get("interpolation") is not None:
        cargs.extend([
            "--interpolation",
            params.get("interpolation")
        ])
    if params.get("restrict_deformation") is not None:
        cargs.extend([
            "-g",
            "x".join(map(str, params.get("restrict_deformation")))
        ])
    if params.get("initial_fixed_transform") is not None:
        cargs.extend([
            "-q",
            params.get("initial_fixed_transform")
        ])
    if params.get("initial_moving_transform") is not None:
        cargs.extend([
            "--initial-moving-transform",
            *dyn_cargs(params.get("initial_moving_transform")["__STYXTYPE__"])(params.get("initial_moving_transform"), execution)
        ])
    cargs.extend([a for c in [dyn_cargs(s["__STYXTYPE__"])(s, execution) for s in params.get("stages")] for a in c])
    if params.get("winsorize_image_intensities") is not None:
        cargs.extend([
            "--winsorize-image-intensities",
            *dyn_cargs(params.get("winsorize_image_intensities")["__STYXTYPE__"])(params.get("winsorize_image_intensities"), execution)
        ])
    if params.get("masks") is not None:
        cargs.extend([
            "--masks",
            *dyn_cargs(params.get("masks")["__STYXTYPE__"])(params.get("masks"), execution)
        ])
    if params.get("minc") is not None:
        cargs.extend([
            "--minc",
            str(params.get("minc"))
        ])
    if params.get("random_seed") is not None:
        cargs.extend([
            "--random-seed",
            str(params.get("random_seed"))
        ])
    if params.get("verbose") is not None:
        cargs.extend([
            "-v",
            str(params.get("verbose"))
        ])
    if params.get("float") is not None:
        cargs.extend([
            "--float",
            str(params.get("float"))
        ])
    return cargs


def ants_registration_outputs(
    params: AntsRegistrationParameters,
    execution: Execution,
) -> AntsRegistrationOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AntsRegistrationOutputs(
        root=execution.output_file("."),
        generic_affine=execution.output_file(params.get("output") + "0GenericAffine.mat") if (params.get("output") is not None) else None,
        inverse_warped=execution.output_file(params.get("output") + "InverseWarped.nii.gz") if (params.get("output") is not None) else None,
        inverse_warp=execution.output_file(params.get("output") + "1InverseWarp.nii.gz") if (params.get("output") is not None) else None,
        warped=execution.output_file(params.get("output") + "Warped.nii.gz") if (params.get("output") is not None) else None,
        warp=execution.output_file(params.get("output") + "1Warp.nii.gz") if (params.get("output") is not None) else None,
    )
    return ret


def ants_registration_execute(
    params: AntsRegistrationParameters,
    execution: Execution,
) -> AntsRegistrationOutputs:
    """
    This program is a user-level registration application meant to utilize classes
    in ITK v4.0 and later. The user can specify any number of "stages" where a stage
    consists of a transform; an image metric; and iterations, shrink factors, and
    smoothing sigmas for each level. Note that explicitly setting the
    dimensionality, metric, transform, output, convergence, shrink-factors, and
    smoothing-sigmas parameters is mandatory.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        execution: The execution object.
    Returns:
        NamedTuple of outputs (described in `AntsRegistrationOutputs`).
    """
    params = execution.params(params)
    cargs = ants_registration_cargs(params, execution)
    ret = ants_registration_outputs(params, execution)
    execution.run(cargs)
    return ret


def ants_registration(
    stages: list[AntsRegistrationStageParameters],
    dimensionality: typing.Literal[2, 3, 4] | None = None,
    output: str | None = None,
    save_state: str | None = None,
    restore_state: str | None = None,
    write_composite_transform: typing.Literal[0, 1] | None = None,
    print_similarity_measure_interval: int | None = None,
    write_interval_volumes: int | None = None,
    collapse_output_transforms: typing.Literal[1, 0] | None = None,
    initialize_transforms_per_stage: typing.Literal[1, 0] | None = None,
    interpolation: typing.Literal["Linear", "NearestNeighbor", "MultiLabel", "Gaussian", "BSpline", "CosineWindowedSinc", "WelchWindowedSinc", "HammingWindowedSinc", "LanczosWindowedSinc", "GenericLabel"] | None = None,
    restrict_deformation: list[typing.Literal[0, 1]] | None = None,
    initial_fixed_transform: str | None = None,
    initial_moving_transform: typing.Union[AntsRegistrationInitialMovingTransformParameters, AntsRegistrationInitialMovingTransformUseInverseParameters, AntsRegistrationInitialMovingTransformInitializationFeatureParameters] | None = None,
    winsorize_image_intensities: AntsRegistrationWinsorizeImageIntensitiesParameters | None = None,
    masks: AntsRegistrationMasksParameters | None = None,
    minc: typing.Literal[0, 1] | None = None,
    random_seed: int | None = None,
    verbose: typing.Literal[0, 1] | None = None,
    float_: typing.Literal[0, 1] | None = None,
    runner: Runner | None = None,
) -> AntsRegistrationOutputs:
    """
    This program is a user-level registration application meant to utilize classes
    in ITK v4.0 and later. The user can specify any number of "stages" where a stage
    consists of a transform; an image metric; and iterations, shrink factors, and
    smoothing sigmas for each level. Note that explicitly setting the
    dimensionality, metric, transform, output, convergence, shrink-factors, and
    smoothing-sigmas parameters is mandatory.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        stages: Stages of the registration process.
        dimensionality: This option forces the image to be treated as a\
            specified-dimensional image. If not specified, we try to infer the\
            dimensionality from the input image.
        output: Specify the output transform prefix (output format is .nii.gz\
            ). Optionally, one can choose to warp the moving image to the fixed\
            space and, if the inverse transform exists, one can also output the\
            warped fixed image. Note that only the images specified in the first\
            metric call are warped. Use antsApplyTransforms to warp other images\
            using the resultant transform(s). When a composite transform is not\
            specified, linear transforms are specified with a '.mat' suffix and\
            displacement fields with a 'Warp.nii.gz' suffix (and\
            'InverseWarp.nii.gz', when applicable. In addition, for velocity-based\
            transforms, the full velocity field is written to file\
            ('VelocityField.nii.gz') as long as the collapse transforms flag is\
            turned off ('-z 0').
        save_state: Specify the output file for the current state of the\
            registration. The state file is written to an hdf5 composite file. It\
            is specially usefull if we want to save the current state of a SyN\
            registration to the disk, so we can load and restore that later to\
            continue the next registration process directly started from the last\
            saved state. The output file of this flag is the same as the\
            write-composite-transform, unless the last transform is a SyN\
            transform. In that case, the inverse displacement field of the SyN\
            transform is also added to the output composite transform. Again notice\
            that this file cannot be treated as a transform, and restore-state\
            option must be used to load the written file by this flag.
        restore_state: Specify the initial state of the registration which get\
            immediately used to directly initialize the registration process. The\
            flag is mutually exclusive with other intialization flags.If this flag\
            is used, none of the initial-moving-transform and\
            initial-fixed-transform cannot be used.
        write_composite_transform: Boolean specifying whether or not the\
            composite transform (and its inverse, if it exists) should be written\
            to an hdf5 composite file. This is false by default so that only the\
            transform for each stage is written to file.
        print_similarity_measure_interval: Prints out the CC similarity metric\
            measure between the full-size input fixed and the transformed moving\
            images at each iteration a value of 0 (the default) indicates that the\
            full scale computation should not take placeany value greater than 0\
            represents the interval of full scale metric computation.
        write_interval_volumes: Writes out the output volume at each iteration.\
            It helps to present the registration process as a short movie a value\
            of 0 (the default) indicates that this option should not take placeany\
            value greater than 0 represents the interval between the iterations\
            which outputs are written to the disk.
        collapse_output_transforms: Collapse output transforms. Specifically,\
            enabling this option combines all adjacent transforms where possible.\
            All adjacent linear transforms are written to disk in the form of an\
            itk affine transform (called xxxGenericAffine.mat).\
            Similarly, all adjacent displacement field transforms are combined\
            when written to disk (e.g. xxxWarp.nii.gz and xxxInverseWarp.nii.gz\
            (if available)). Also, an output composite transform including the\
            collapsed transforms is written to the disk (called\
            outputCollapsed(Inverse)Composite).
        initialize_transforms_per_stage: Initialize linear transforms from the\
            previous stage. By enabling this option, the current linear stage\
            transform is directly intialized from the previous stage's linear\
            transform; this allows multiple linear stages to be run where each\
            stage directly updates the estimated linear transform from the previous\
            stage. (e.g. Translation -> Rigid -> Affine).
        interpolation: Several interpolation options are available in ITK.\
            These have all been made available. Currently the interpolator choice\
            is only used to warp (and possibly inverse warp) the final output\
            image(s).
        restrict_deformation: This option allows the user to restrict the\
            optimization of the displacement field, translation, rigid or affine\
            transform on a per-component basis. For example, if one wants to limit\
            the deformation or rotation of 3-D volume to the first two dimensions,\
            this is possible by specifying a weight vector of '1x1x0' for a\
            deformation field or '1x1x0x1x1x0' for a rigid transformation.\
            Low-dimensional restriction only works if there are no preceding\
            transformations.All stages up to and including the desired stage must\
            have this option specified,even if they should not be restricted (in\
            which case specify 1x1x1...).
        initial_fixed_transform: Specify the initial fixed transform(s) which\
            get immediately incorporated into the composite transform. The order of\
            the transforms is stack-esque in that the last transform specified on\
            the command line is the first to be applied. In addition to\
            initialization with ITK transforms, the user can perform an initial\
            translation alignment by specifying the fixed and moving images and\
            selecting an initialization feature. These features include using the\
            geometric center of the images (=0), the image intensities (=1), or the\
            origin of the images (=2).
        initial_moving_transform: Specify the initial moving transform(s) which\
            get immediately incorporated into the composite transform. The order of\
            the transforms is stack-esque in that the last transform specified on\
            the command line is the first to be applied. In addition to\
            initialization with ITK transforms, the user can perform an initial\
            translation alignment by specifying the fixed and moving images and\
            selecting an initialization feature. These features include using the\
            geometric center of the images (=0), the image intensities (=1), or the\
            origin of the images (=2).
        winsorize_image_intensities: Winsorize data based on specified\
            quantiles.
        masks: Image masks to limit voxels considered by the metric. Two\
            options are allowed for mask specification: 1) Either the user\
            specifies a single mask to be used for all stages or 2) the user\
            specifies a mask for each stage. With the latter one can select to\
            which stages masks are applied by supplying valid file names. If the\
            file does not exist, a mask will not be used for that stage. Note that\
            we handle the fixed and moving masks separately to enforce this\
            constraint.
        minc: Use MINC file formats for transformations.
        random_seed: Random seed.
        verbose: Verbose output.
        float_: Use 'float' instead of 'double' for computations.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsRegistrationOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTS_REGISTRATION_METADATA)
    params = ants_registration_params(
        dimensionality=dimensionality,
        output=output,
        save_state=save_state,
        restore_state=restore_state,
        write_composite_transform=write_composite_transform,
        print_similarity_measure_interval=print_similarity_measure_interval,
        write_interval_volumes=write_interval_volumes,
        collapse_output_transforms=collapse_output_transforms,
        initialize_transforms_per_stage=initialize_transforms_per_stage,
        interpolation=interpolation,
        restrict_deformation=restrict_deformation,
        initial_fixed_transform=initial_fixed_transform,
        initial_moving_transform=initial_moving_transform,
        stages=stages,
        winsorize_image_intensities=winsorize_image_intensities,
        masks=masks,
        minc=minc,
        random_seed=random_seed,
        verbose=verbose,
        float_=float_,
    )
    return ants_registration_execute(params, execution)


__all__ = [
    "ANTS_REGISTRATION_METADATA",
    "AntsRegistrationConvergenceParameters",
    "AntsRegistrationInitialMovingTransformInitializationFeatureParameters",
    "AntsRegistrationInitialMovingTransformParameters",
    "AntsRegistrationInitialMovingTransformUseInverseParameters",
    "AntsRegistrationMasksParameters",
    "AntsRegistrationMetricAntsNeighbourhoodCrossCorrelationParameters",
    "AntsRegistrationMetricDemonsParameters",
    "AntsRegistrationMetricEuclideanIcpParameters",
    "AntsRegistrationMetricGlobalCorrelationParameters",
    "AntsRegistrationMetricIgdmParameters",
    "AntsRegistrationMetricJensenHavrdaCharvetTsallisParameters",
    "AntsRegistrationMetricMattesParameters",
    "AntsRegistrationMetricMeanSquaresParameters",
    "AntsRegistrationMetricMutualInformationParameters",
    "AntsRegistrationMetricPointSetExpectationParameters",
    "AntsRegistrationOutputs",
    "AntsRegistrationParameters",
    "AntsRegistrationStageParameters",
    "AntsRegistrationTransformAffineParameters",
    "AntsRegistrationTransformBsplineDisplacementFieldParameters",
    "AntsRegistrationTransformBsplineExponentialParameters",
    "AntsRegistrationTransformBsplineParameters",
    "AntsRegistrationTransformBsplineSynParameters",
    "AntsRegistrationTransformCompositeAffineParameters",
    "AntsRegistrationTransformExponentialParameters",
    "AntsRegistrationTransformGaussianDisplacementFieldParameters",
    "AntsRegistrationTransformRigidParameters",
    "AntsRegistrationTransformSimilarityParameters",
    "AntsRegistrationTransformSynParameters",
    "AntsRegistrationTransformTimeVaryingBsplineVelocityFieldParameters",
    "AntsRegistrationTransformTimeVaryingVelocityFieldParameters",
    "AntsRegistrationTransformTranslationParameters",
    "AntsRegistrationWinsorizeImageIntensitiesParameters",
    "ants_registration",
    "ants_registration_convergence_params",
    "ants_registration_initial_moving_transform_initialization_feature_params",
    "ants_registration_initial_moving_transform_params",
    "ants_registration_initial_moving_transform_use_inverse_params",
    "ants_registration_masks_params",
    "ants_registration_metric_ants_neighbourhood_cross_correlation_params",
    "ants_registration_metric_demons_params",
    "ants_registration_metric_euclidean_icp_params",
    "ants_registration_metric_global_correlation_params",
    "ants_registration_metric_igdm_params",
    "ants_registration_metric_jensen_havrda_charvet_tsallis_params",
    "ants_registration_metric_mattes_params",
    "ants_registration_metric_mean_squares_params",
    "ants_registration_metric_mutual_information_params",
    "ants_registration_metric_point_set_expectation_params",
    "ants_registration_params",
    "ants_registration_stage_params",
    "ants_registration_transform_affine_params",
    "ants_registration_transform_bspline_displacement_field_params",
    "ants_registration_transform_bspline_exponential_params",
    "ants_registration_transform_bspline_params",
    "ants_registration_transform_bspline_syn_params",
    "ants_registration_transform_composite_affine_params",
    "ants_registration_transform_exponential_params",
    "ants_registration_transform_gaussian_displacement_field_params",
    "ants_registration_transform_rigid_params",
    "ants_registration_transform_similarity_params",
    "ants_registration_transform_syn_params",
    "ants_registration_transform_time_varying_bspline_velocity_field_params",
    "ants_registration_transform_time_varying_velocity_field_params",
    "ants_registration_transform_translation_params",
    "ants_registration_winsorize_image_intensities_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

ANTS_MULTIVARIATE_TEMPLATE_CONSTRUCTION2_SH_METADATA = Metadata(
    id="b866ae2f76511eca887345fe0515e30b2318f31e.boutiques",
    name="antsMultivariateTemplateConstruction2.sh",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


AntsMultivariateTemplateConstruction2ShParameters = typing.TypedDict('AntsMultivariateTemplateConstruction2ShParameters', {
    "__STYX_TYPE__": typing.Literal["antsMultivariateTemplateConstruction2.sh"],
    "input": str,
})


def dyn_cargs(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "antsMultivariateTemplateConstruction2.sh": ants_multivariate_template_construction2_sh_cargs,
    }.get(t)


def dyn_outputs(
    t: str,
) -> typing.Any:
    """
    Get build outputs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build outputs function.
    """
    return {
        "antsMultivariateTemplateConstruction2.sh": ants_multivariate_template_construction2_sh_outputs,
    }.get(t)


class AntsMultivariateTemplateConstruction2ShOutputs(typing.NamedTuple):
    """
    Output object returned when calling `ants_multivariate_template_construction2_sh(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    template: OutputPathType
    """The output multivariate template."""


def ants_multivariate_template_construction2_sh_params(
    input_: str,
) -> AntsMultivariateTemplateConstruction2ShParameters:
    """
    Build parameters.
    
    Args:
        input_: Options for setting up and running the multivariate template\
            construction process.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "antsMultivariateTemplateConstruction2.sh",
        "input": input_,
    }
    return params


def ants_multivariate_template_construction2_sh_cargs(
    params: AntsMultivariateTemplateConstruction2ShParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("antsMultivariateTemplateConstruction2.sh")
    cargs.append(params.get("input"))
    return cargs


def ants_multivariate_template_construction2_sh_outputs(
    params: AntsMultivariateTemplateConstruction2ShParameters,
    execution: Execution,
) -> AntsMultivariateTemplateConstruction2ShOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AntsMultivariateTemplateConstruction2ShOutputs(
        root=execution.output_file("."),
        template=execution.output_file("<output-prefix>Template.nii.gz"),
    )
    return ret


def ants_multivariate_template_construction2_sh_execute(
    params: AntsMultivariateTemplateConstruction2ShParameters,
    execution: Execution,
) -> AntsMultivariateTemplateConstruction2ShOutputs:
    """
    The antsMultivariateTemplateConstruction2.sh script is part of the Advanced
    Normalization Tools (ANTs) suite. It is used for constructing multivariate
    templates.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        execution: The execution object.
    Returns:
        NamedTuple of outputs (described in `AntsMultivariateTemplateConstruction2ShOutputs`).
    """
    params = execution.params(params)
    cargs = ants_multivariate_template_construction2_sh_cargs(params, execution)
    ret = ants_multivariate_template_construction2_sh_outputs(params, execution)
    execution.run(cargs)
    return ret


def ants_multivariate_template_construction2_sh(
    input_: str,
    runner: Runner | None = None,
) -> AntsMultivariateTemplateConstruction2ShOutputs:
    """
    The antsMultivariateTemplateConstruction2.sh script is part of the Advanced
    Normalization Tools (ANTs) suite. It is used for constructing multivariate
    templates.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        input_: Options for setting up and running the multivariate template\
            construction process.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsMultivariateTemplateConstruction2ShOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTS_MULTIVARIATE_TEMPLATE_CONSTRUCTION2_SH_METADATA)
    params = ants_multivariate_template_construction2_sh_params(
        input_=input_,
    )
    return ants_multivariate_template_construction2_sh_execute(params, execution)


__all__ = [
    "ANTS_MULTIVARIATE_TEMPLATE_CONSTRUCTION2_SH_METADATA",
    "AntsMultivariateTemplateConstruction2ShOutputs",
    "AntsMultivariateTemplateConstruction2ShParameters",
    "ants_multivariate_template_construction2_sh",
    "ants_multivariate_template_construction2_sh_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CONVERT_TRANSFORM_FILE_METADATA = Metadata(
    id="d418439c73661bae3620741d30dc78169c5a445e.boutiques",
    name="ConvertTransformFile",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


ConvertTransformFileParameters = typing.TypedDict('ConvertTransformFileParameters', {
    "__STYX_TYPE__": typing.Literal["ConvertTransformFile"],
    "dimensions": int,
    "input_transform_file": InputPathType,
    "output_transform_file": str,
    "matrix": bool,
    "homogeneous_matrix": bool,
    "RAS": bool,
    "convert_to_affine_type": bool,
})


def dyn_cargs(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "ConvertTransformFile": convert_transform_file_cargs,
    }.get(t)


def dyn_outputs(
    t: str,
) -> typing.Any:
    """
    Get build outputs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build outputs function.
    """
    return {
    }.get(t)


class ConvertTransformFileOutputs(typing.NamedTuple):
    """
    Output object returned when calling `convert_transform_file(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def convert_transform_file_params(
    dimensions: int,
    input_transform_file: InputPathType,
    output_transform_file: str,
    matrix: bool = False,
    homogeneous_matrix: bool = False,
    ras: bool = False,
    convert_to_affine_type: bool = False,
) -> ConvertTransformFileParameters:
    """
    Build parameters.
    
    Args:
        dimensions: Dimensionality of the transform file.
        input_transform_file: Path to the input transform file.
        output_transform_file: Path for the output transform file.
        matrix: Output only the transform matrix to a text file, one row per\
            line with space-delimited values. Only works for specific transform\
            types.
        homogeneous_matrix: Output an N+1 square homogeneous matrix from the\
            transform matrix and offset. Only works for specific transform types.
        ras: Convert the output into the RAS coordinate system if combined with\
            'matrix' or 'homogeneousMatrix'.
        convert_to_affine_type: Convert the input transform type to\
            AffineTransform and output again as a binary transform file.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "ConvertTransformFile",
        "dimensions": dimensions,
        "input_transform_file": input_transform_file,
        "output_transform_file": output_transform_file,
        "matrix": matrix,
        "homogeneous_matrix": homogeneous_matrix,
        "RAS": ras,
        "convert_to_affine_type": convert_to_affine_type,
    }
    return params


def convert_transform_file_cargs(
    params: ConvertTransformFileParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("ConvertTransformFile")
    cargs.append(str(params.get("dimensions")))
    cargs.append(execution.input_file(params.get("input_transform_file")))
    cargs.append(params.get("output_transform_file"))
    if params.get("matrix"):
        cargs.append("--matrix")
    if params.get("homogeneous_matrix"):
        cargs.append("--homogeneousMatrix")
    if params.get("RAS"):
        cargs.append("--RAS")
    if params.get("convert_to_affine_type"):
        cargs.append("--convertToAffineType")
    return cargs


def convert_transform_file_outputs(
    params: ConvertTransformFileParameters,
    execution: Execution,
) -> ConvertTransformFileOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = ConvertTransformFileOutputs(
        root=execution.output_file("."),
    )
    return ret


def convert_transform_file_execute(
    params: ConvertTransformFileParameters,
    execution: Execution,
) -> ConvertTransformFileOutputs:
    """
    Utility to read in a transform file (presumed to be in binary format) and output
    it in various formats. Default output is legacy human-readable text format.
    Without any options, the output filename extension must be .txt or .tfm to
    signify a text-formatted transform file.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        execution: The execution object.
    Returns:
        NamedTuple of outputs (described in `ConvertTransformFileOutputs`).
    """
    params = execution.params(params)
    cargs = convert_transform_file_cargs(params, execution)
    ret = convert_transform_file_outputs(params, execution)
    execution.run(cargs)
    return ret


def convert_transform_file(
    dimensions: int,
    input_transform_file: InputPathType,
    output_transform_file: str,
    matrix: bool = False,
    homogeneous_matrix: bool = False,
    ras: bool = False,
    convert_to_affine_type: bool = False,
    runner: Runner | None = None,
) -> ConvertTransformFileOutputs:
    """
    Utility to read in a transform file (presumed to be in binary format) and output
    it in various formats. Default output is legacy human-readable text format.
    Without any options, the output filename extension must be .txt or .tfm to
    signify a text-formatted transform file.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        dimensions: Dimensionality of the transform file.
        input_transform_file: Path to the input transform file.
        output_transform_file: Path for the output transform file.
        matrix: Output only the transform matrix to a text file, one row per\
            line with space-delimited values. Only works for specific transform\
            types.
        homogeneous_matrix: Output an N+1 square homogeneous matrix from the\
            transform matrix and offset. Only works for specific transform types.
        ras: Convert the output into the RAS coordinate system if combined with\
            'matrix' or 'homogeneousMatrix'.
        convert_to_affine_type: Convert the input transform type to\
            AffineTransform and output again as a binary transform file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ConvertTransformFileOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(CONVERT_TRANSFORM_FILE_METADATA)
    params = convert_transform_file_params(
        dimensions=dimensions,
        input_transform_file=input_transform_file,
        output_transform_file=output_transform_file,
        matrix=matrix,
        homogeneous_matrix=homogeneous_matrix,
        ras=ras,
        convert_to_affine_type=convert_to_affine_type,
    )
    return convert_transform_file_execute(params, execution)


__all__ = [
    "CONVERT_TRANSFORM_FILE_METADATA",
    "ConvertTransformFileOutputs",
    "ConvertTransformFileParameters",
    "convert_transform_file",
    "convert_transform_file_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

ANTS_APPLY_TRANSFORMS_TO_POINTS_METADATA = Metadata(
    id="1ba8745cbb78f985d998ca462d13cc2627bbcd68.boutiques",
    name="antsApplyTransformsToPoints",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


AntsApplyTransformsToPointsSingleTransformParameters = typing.TypedDict('AntsApplyTransformsToPointsSingleTransformParameters', {
    "__STYX_TYPE__": typing.Literal["single_transform"],
})


AntsApplyTransformsToPointsInverseTransformParameters = typing.TypedDict('AntsApplyTransformsToPointsInverseTransformParameters', {
    "__STYX_TYPE__": typing.Literal["inverse_transform"],
    "transform_file": InputPathType,
})


AntsApplyTransformsToPointsParameters = typing.TypedDict('AntsApplyTransformsToPointsParameters', {
    "__STYX_TYPE__": typing.Literal["antsApplyTransformsToPoints"],
    "dimensionality": typing.NotRequired[typing.Literal[2, 3] | None],
    "precision": typing.NotRequired[typing.Literal[0, 1] | None],
    "forantsr": typing.NotRequired[typing.Literal[0, 1] | None],
    "input": InputPathType,
    "output": str,
    "transform": typing.NotRequired[typing.Union[AntsApplyTransformsToPointsSingleTransformParameters, AntsApplyTransformsToPointsInverseTransformParameters] | None],
})


def dyn_cargs(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "antsApplyTransformsToPoints": ants_apply_transforms_to_points_cargs,
        "single_transform": ants_apply_transforms_to_points_single_transform_cargs,
        "inverse_transform": ants_apply_transforms_to_points_inverse_transform_cargs,
    }.get(t)


def dyn_outputs(
    t: str,
) -> typing.Any:
    """
    Get build outputs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build outputs function.
    """
    return {
        "antsApplyTransformsToPoints": ants_apply_transforms_to_points_outputs,
    }.get(t)


def ants_apply_transforms_to_points_single_transform_params(
) -> AntsApplyTransformsToPointsSingleTransformParameters:
    """
    Build parameters.
    
    Args:
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "single_transform",
    }
    return params


def ants_apply_transforms_to_points_single_transform_cargs(
    params: AntsApplyTransformsToPointsSingleTransformParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("[TRANSFORM]")
    return cargs


def ants_apply_transforms_to_points_inverse_transform_params(
    transform_file: InputPathType,
) -> AntsApplyTransformsToPointsInverseTransformParameters:
    """
    Build parameters.
    
    Args:
        transform_file:.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "inverse_transform",
        "transform_file": transform_file,
    }
    return params


def ants_apply_transforms_to_points_inverse_transform_cargs(
    params: AntsApplyTransformsToPointsInverseTransformParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(execution.input_file(params.get("transform_file")) + ",1")
    return cargs


class AntsApplyTransformsToPointsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `ants_apply_transforms_to_points(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    warped_points: OutputPathType
    """The output is the CSV file containing warped points."""


def ants_apply_transforms_to_points_params(
    input_: InputPathType,
    output: str,
    dimensionality: typing.Literal[2, 3] | None = None,
    precision: typing.Literal[0, 1] | None = None,
    forantsr: typing.Literal[0, 1] | None = None,
    transform: typing.Union[AntsApplyTransformsToPointsSingleTransformParameters, AntsApplyTransformsToPointsInverseTransformParameters] | None = None,
) -> AntsApplyTransformsToPointsParameters:
    """
    Build parameters.
    
    Args:
        input_: Input CSV file with columns including x,y,z,t headers that\
            define the points in physical space, or a 2D .mha binary image file.
        output: Output the warped points to a CSV file.
        dimensionality: This option forces the points to be treated as a\
            specified-dimensionality.
        precision: Use double precision.
        forantsr: Set true for ANTsR IO.
        transform: Transform file(s) to apply to the input points. Uses an\
            inverse transform if specified as [transformFileName,1].
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "antsApplyTransformsToPoints",
        "input": input_,
        "output": output,
    }
    if dimensionality is not None:
        params["dimensionality"] = dimensionality
    if precision is not None:
        params["precision"] = precision
    if forantsr is not None:
        params["forantsr"] = forantsr
    if transform is not None:
        params["transform"] = transform
    return params


def ants_apply_transforms_to_points_cargs(
    params: AntsApplyTransformsToPointsParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("antsApplyTransformsToPoints")
    if params.get("dimensionality") is not None:
        cargs.extend([
            "--dimensionality",
            str(params.get("dimensionality"))
        ])
    if params.get("precision") is not None:
        cargs.extend([
            "--precision",
            str(params.get("precision"))
        ])
    if params.get("forantsr") is not None:
        cargs.extend([
            "--forantsr",
            str(params.get("forantsr"))
        ])
    cargs.extend([
        "-i",
        execution.input_file(params.get("input"))
    ])
    cargs.extend([
        "-o",
        params.get("output")
    ])
    if params.get("transform") is not None:
        cargs.extend([
            "-t",
            *dyn_cargs(params.get("transform")["__STYXTYPE__"])(params.get("transform"), execution)
        ])
    return cargs


def ants_apply_transforms_to_points_outputs(
    params: AntsApplyTransformsToPointsParameters,
    execution: Execution,
) -> AntsApplyTransformsToPointsOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AntsApplyTransformsToPointsOutputs(
        root=execution.output_file("."),
        warped_points=execution.output_file(params.get("output")),
    )
    return ret


def ants_apply_transforms_to_points_execute(
    params: AntsApplyTransformsToPointsParameters,
    execution: Execution,
) -> AntsApplyTransformsToPointsOutputs:
    """
    antsApplyTransformsToPoints, applied to an input image, transforms it according
    to a reference image and a transform (or a set of transforms). It reads in a CSV
    file with the first D columns defining the spatial location where the spatial
    location is defined in physical coordinates. The CSV file should have a header
    row.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        execution: The execution object.
    Returns:
        NamedTuple of outputs (described in `AntsApplyTransformsToPointsOutputs`).
    """
    params = execution.params(params)
    cargs = ants_apply_transforms_to_points_cargs(params, execution)
    ret = ants_apply_transforms_to_points_outputs(params, execution)
    execution.run(cargs)
    return ret


def ants_apply_transforms_to_points(
    input_: InputPathType,
    output: str,
    dimensionality: typing.Literal[2, 3] | None = None,
    precision: typing.Literal[0, 1] | None = None,
    forantsr: typing.Literal[0, 1] | None = None,
    transform: typing.Union[AntsApplyTransformsToPointsSingleTransformParameters, AntsApplyTransformsToPointsInverseTransformParameters] | None = None,
    runner: Runner | None = None,
) -> AntsApplyTransformsToPointsOutputs:
    """
    antsApplyTransformsToPoints, applied to an input image, transforms it according
    to a reference image and a transform (or a set of transforms). It reads in a CSV
    file with the first D columns defining the spatial location where the spatial
    location is defined in physical coordinates. The CSV file should have a header
    row.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        input_: Input CSV file with columns including x,y,z,t headers that\
            define the points in physical space, or a 2D .mha binary image file.
        output: Output the warped points to a CSV file.
        dimensionality: This option forces the points to be treated as a\
            specified-dimensionality.
        precision: Use double precision.
        forantsr: Set true for ANTsR IO.
        transform: Transform file(s) to apply to the input points. Uses an\
            inverse transform if specified as [transformFileName,1].
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsApplyTransformsToPointsOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTS_APPLY_TRANSFORMS_TO_POINTS_METADATA)
    params = ants_apply_transforms_to_points_params(
        dimensionality=dimensionality,
        precision=precision,
        forantsr=forantsr,
        input_=input_,
        output=output,
        transform=transform,
    )
    return ants_apply_transforms_to_points_execute(params, execution)


__all__ = [
    "ANTS_APPLY_TRANSFORMS_TO_POINTS_METADATA",
    "AntsApplyTransformsToPointsInverseTransformParameters",
    "AntsApplyTransformsToPointsOutputs",
    "AntsApplyTransformsToPointsParameters",
    "AntsApplyTransformsToPointsSingleTransformParameters",
    "ants_apply_transforms_to_points",
    "ants_apply_transforms_to_points_inverse_transform_params",
    "ants_apply_transforms_to_points_params",
    "ants_apply_transforms_to_points_single_transform_params",
]



"""
ANTs

Advanced Normalization Tools (ANTs) is a C++ library available through the
command line that computes high-dimensional mappings to capture the statistics
of brain structure and function. It allows one to organize, visualize and
statistically explore large biomedical image sets. Additionally, it integrates
imaging modalities in space + time and works across species or organ systems
with minimal customization.

The ANTs library is considered a state-of-the-art medical image registration and
segmentation toolkit which depends on the Insight ToolKit, a widely used medical
image processing library to which ANTs developers contribute. ANTs-related tools
have also won several international, unbiased competitions such as MICCAI,
BRATS, and STACOM.

URL: https://github.com/ANTsX/ANTs
"""
# This file was auto generated by Styx.
# Do not edit this file directly.

from .add_noise_to_image import *
from .ants_ai import *
from .ants_align_origin import *
from .ants_apply_transforms import *
from .ants_apply_transforms_to_points import *
from .ants_atropos_n4_sh import *
from .ants_brain_extraction_sh import *
from .ants_cortical_thickness_sh import *
from .ants_intermodality_intrasubject_sh import *
from .ants_introduction_sh import *
from .ants_joint_fusion import *
from .ants_joint_label_fusion_sh import *
from .ants_joint_tensor_fusion import *
from .ants_landmark_based_transform_initializer import *
from .ants_motion_corr import *
from .ants_motion_corr_diffusion_direction import *
from .ants_motion_corr_stats import *
from .ants_multivariate_template_construction2_sh import *
from .ants_neuroimaging_battery import *
from .ants_registration import *
from .ants_registration_sy_n_sh import *
from .ants_registration_sy_nquick_sh import *
from .ants_slice_regularized_registration import *
from .ants_transform_info import *
from .antsintegrate_vector_field import *
from .antsjacobian import *
from .antsuse_deformation_field_to_get_affine_transform import *
from .antsuse_landmark_images_to_get_affine_transform import *
from .antsuse_landmark_images_to_get_bspline_displacement_field import *
from .atropos import *
from .convert_scalar_image_to_rgb import *
from .convert_to_jpg import *
from .convert_transform_file import *
from .create_displacement_field import *
from .create_dticohort import *
from .create_tiled_mosaic import *
from .create_warped_grid_image import *
from .denoise_image import *
from .extract_region_from_image import *
from .extract_region_from_image_by_mask import *
from .i_math import *
from .image_intensity_statistics import *
from .image_math import *
from .image_set_statistics import *
from .kelly_kapowski import *
from .label_geometry_measures import *
from .lesion_filling import *
from .multiply_images import *
from .n3_bias_field_correction import *
from .n4_bias_field_correction import *
from .non_local_super_resolution import *
from .paste_image_into_image import *
from .print_header import *
from .rebase_tensor_image import *
from .resample_image import *
from .sccan import *
from .set_spacing import *
from .simple_syn_registration import *
from .simulate_displacement_field import *
from .smooth_displacement_field import *
from .smooth_image import *
from .super_resolution import *
from .surface_based_smoothing import *
from .surface_curvature import *
from .texture_cooccurrence_features import *
from .texture_run_length_features import *
from .threshold_image import *
from .tile_images import *
from .time_sccan import *
from .warp_tensor_image_multi_transform import *
from .warp_time_series_image_multi_transform import *



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

ANTS_SLICE_REGULARIZED_REGISTRATION_METADATA = Metadata(
    id="f5c4948fa2560385a828a453a54af7e3a4ab4be7.boutiques",
    name="antsSliceRegularizedRegistration",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


AntsSliceRegularizedRegistrationParameters = typing.TypedDict('AntsSliceRegularizedRegistrationParameters', {
    "__STYX_TYPE__": typing.Literal["antsSliceRegularizedRegistration"],
    "polydegree": int,
    "output": str,
    "metric": str,
    "transform": str,
    "iterations": str,
    "shrink_factors": str,
    "smoothing_sigmas": str,
    "mask": typing.NotRequired[InputPathType | None],
    "interpolation": typing.NotRequired[typing.Literal["Linear", "NearestNeighbor", "MultiLabel", "Gaussian", "BSpline", "CosineWindowedSinc", "WelchWindowedSinc", "HammingWindowedSinc", "LanczosWindowedSinc", "GenericLabel"] | None],
    "verbose": typing.NotRequired[typing.Literal[0] | None],
})


def dyn_cargs(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "antsSliceRegularizedRegistration": ants_slice_regularized_registration_cargs,
    }.get(t)


def dyn_outputs(
    t: str,
) -> typing.Any:
    """
    Get build outputs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build outputs function.
    """
    return {
        "antsSliceRegularizedRegistration": ants_slice_regularized_registration_outputs,
    }.get(t)


class AntsSliceRegularizedRegistrationOutputs(typing.NamedTuple):
    """
    Output object returned when calling `ants_slice_regularized_registration(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    polynomial_fit: OutputPathType
    """Output is the polynomial fit to Tx & Ty."""
    transformed_image: OutputPathType
    """Output is the transformed image."""


def ants_slice_regularized_registration_params(
    polydegree: int,
    output: str,
    metric: str,
    transform: str,
    iterations: str,
    shrink_factors: str,
    smoothing_sigmas: str,
    mask: InputPathType | None = None,
    interpolation: typing.Literal["Linear", "NearestNeighbor", "MultiLabel", "Gaussian", "BSpline", "CosineWindowedSinc", "WelchWindowedSinc", "HammingWindowedSinc", "LanczosWindowedSinc", "GenericLabel"] | None = None,
    verbose: typing.Literal[0] | None = None,
) -> AntsSliceRegularizedRegistrationParameters:
    """
    Build parameters.
    
    Args:
        polydegree: Degree of polynomial up to zDimension-2. Controls the\
            polynomial degree. 0 means no regularization.
        output: Specify the output transform prefix (output format is .nii.gz).\
            Optionally, one can choose to warp the moving image to the fixed space,\
            and if the inverse transform exists, one can also output the warped\
            fixed image.
        metric: Four image metrics are available: GC: global correlation, CC:\
            ANTS neighborhood cross correlation, MI: Mutual information, and\
            MeanSquares: mean-squares intensity difference.
        transform: Several transform options are available. The gradientStep or\
            learningRate characterizes the gradient descent optimization.
        iterations: Specify the number of iterations at each level.
        shrink_factors: Specify the shrink factor for the virtual domain\
            (typically the fixed image) at each level.
        smoothing_sigmas: Specify the amount of smoothing at each level.
        mask: Fixed image mask to limit voxels considered by the metric.
        interpolation: Several interpolation options are available in ITK.
        verbose: Verbose option.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "antsSliceRegularizedRegistration",
        "polydegree": polydegree,
        "output": output,
        "metric": metric,
        "transform": transform,
        "iterations": iterations,
        "shrink_factors": shrink_factors,
        "smoothing_sigmas": smoothing_sigmas,
    }
    if mask is not None:
        params["mask"] = mask
    if interpolation is not None:
        params["interpolation"] = interpolation
    if verbose is not None:
        params["verbose"] = verbose
    return params


def ants_slice_regularized_registration_cargs(
    params: AntsSliceRegularizedRegistrationParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("antsSliceRegularizedRegistration")
    cargs.extend([
        "-p",
        str(params.get("polydegree"))
    ])
    cargs.extend([
        "-o",
        params.get("output")
    ])
    cargs.extend([
        "-m",
        params.get("metric")
    ])
    cargs.extend([
        "-t",
        params.get("transform")
    ])
    cargs.extend([
        "-i",
        params.get("iterations")
    ])
    cargs.extend([
        "-f",
        params.get("shrink_factors")
    ])
    cargs.extend([
        "-s",
        params.get("smoothing_sigmas")
    ])
    if params.get("mask") is not None:
        cargs.extend([
            "-x",
            execution.input_file(params.get("mask"))
        ])
    if params.get("interpolation") is not None:
        cargs.extend([
            "-n",
            params.get("interpolation")
        ])
    if params.get("verbose") is not None:
        cargs.extend([
            "-v",
            str(params.get("verbose"))
        ])
    return cargs


def ants_slice_regularized_registration_outputs(
    params: AntsSliceRegularizedRegistrationParameters,
    execution: Execution,
) -> AntsSliceRegularizedRegistrationOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AntsSliceRegularizedRegistrationOutputs(
        root=execution.output_file("."),
        polynomial_fit=execution.output_file("[OUTPUT_PREFIX]TxTy_poly.csv"),
        transformed_image=execution.output_file("[OUTPUT_PREFIX].nii.gz"),
    )
    return ret


def ants_slice_regularized_registration_execute(
    params: AntsSliceRegularizedRegistrationParameters,
    execution: Execution,
) -> AntsSliceRegularizedRegistrationOutputs:
    """
    This program is a user-level application for slice-by-slice translation
    registration. Results are regularized in z using polynomial regression. The
    program is targeted at spinal cord MRI. Only one stage is supported where a
    stage consists of a transform; an image metric; and iterations, shrink factors,
    and smoothing sigmas for each level. Specialized for 3D data: fixed image is 3D,
    moving image is 3D. Registration is performed slice-by-slice then regularized in
    z. The parameter -p controls the polynomial degree. -p 0 means no
    regularization.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        execution: The execution object.
    Returns:
        NamedTuple of outputs (described in `AntsSliceRegularizedRegistrationOutputs`).
    """
    params = execution.params(params)
    cargs = ants_slice_regularized_registration_cargs(params, execution)
    ret = ants_slice_regularized_registration_outputs(params, execution)
    execution.run(cargs)
    return ret


def ants_slice_regularized_registration(
    polydegree: int,
    output: str,
    metric: str,
    transform: str,
    iterations: str,
    shrink_factors: str,
    smoothing_sigmas: str,
    mask: InputPathType | None = None,
    interpolation: typing.Literal["Linear", "NearestNeighbor", "MultiLabel", "Gaussian", "BSpline", "CosineWindowedSinc", "WelchWindowedSinc", "HammingWindowedSinc", "LanczosWindowedSinc", "GenericLabel"] | None = None,
    verbose: typing.Literal[0] | None = None,
    runner: Runner | None = None,
) -> AntsSliceRegularizedRegistrationOutputs:
    """
    This program is a user-level application for slice-by-slice translation
    registration. Results are regularized in z using polynomial regression. The
    program is targeted at spinal cord MRI. Only one stage is supported where a
    stage consists of a transform; an image metric; and iterations, shrink factors,
    and smoothing sigmas for each level. Specialized for 3D data: fixed image is 3D,
    moving image is 3D. Registration is performed slice-by-slice then regularized in
    z. The parameter -p controls the polynomial degree. -p 0 means no
    regularization.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        polydegree: Degree of polynomial up to zDimension-2. Controls the\
            polynomial degree. 0 means no regularization.
        output: Specify the output transform prefix (output format is .nii.gz).\
            Optionally, one can choose to warp the moving image to the fixed space,\
            and if the inverse transform exists, one can also output the warped\
            fixed image.
        metric: Four image metrics are available: GC: global correlation, CC:\
            ANTS neighborhood cross correlation, MI: Mutual information, and\
            MeanSquares: mean-squares intensity difference.
        transform: Several transform options are available. The gradientStep or\
            learningRate characterizes the gradient descent optimization.
        iterations: Specify the number of iterations at each level.
        shrink_factors: Specify the shrink factor for the virtual domain\
            (typically the fixed image) at each level.
        smoothing_sigmas: Specify the amount of smoothing at each level.
        mask: Fixed image mask to limit voxels considered by the metric.
        interpolation: Several interpolation options are available in ITK.
        verbose: Verbose option.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsSliceRegularizedRegistrationOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTS_SLICE_REGULARIZED_REGISTRATION_METADATA)
    params = ants_slice_regularized_registration_params(
        polydegree=polydegree,
        output=output,
        metric=metric,
        transform=transform,
        iterations=iterations,
        shrink_factors=shrink_factors,
        smoothing_sigmas=smoothing_sigmas,
        mask=mask,
        interpolation=interpolation,
        verbose=verbose,
    )
    return ants_slice_regularized_registration_execute(params, execution)


__all__ = [
    "ANTS_SLICE_REGULARIZED_REGISTRATION_METADATA",
    "AntsSliceRegularizedRegistrationOutputs",
    "AntsSliceRegularizedRegistrationParameters",
    "ants_slice_regularized_registration",
    "ants_slice_regularized_registration_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

ANTS_AI_METADATA = Metadata(
    id="694c77fb3122e7ae7609dc66837caa7834538472.boutiques",
    name="antsAI",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


AntsAiParameters = typing.TypedDict('AntsAiParameters', {
    "__STYX_TYPE__": typing.Literal["antsAI"],
    "dimensionality": typing.NotRequired[typing.Literal[2, 3] | None],
    "metric": typing.Literal["Mattes[fixedImage,movingImage]", "GC[fixedImage,movingImage]", "MI[fixedImage,movingImage]"],
    "transform": typing.Literal["Rigid[gradientStep]", "Affine[gradientStep]", "Similarity[gradientStep]", "AlignGeometricCenters", "AlignCentersOfMass"],
    "align_principal_axes": typing.NotRequired[typing.Literal[0, 1] | None],
    "align_blobs": typing.NotRequired[typing.Literal["numberOfBlobsToExtract", "[numberOfBlobsToExtract,numberOfBlobsToMatch]"] | None],
    "search_factor": typing.NotRequired[typing.Literal["searchFactor", "[searchFactor,arcFraction]"] | None],
    "translation_search_grid": typing.NotRequired[typing.Literal["[stepSize, AxBxC]"] | None],
    "convergence": typing.NotRequired[typing.Literal["numberOfIterations", "[numberOfIterations,convergenceThreshold,convergenceWindowSize]"] | None],
    "masks": typing.NotRequired[typing.Literal["fixedImageMask", "[fixedImageMask,movingImageMask]"] | None],
    "output": str,
    "random_seed": typing.NotRequired[int | None],
    "verbose": typing.NotRequired[typing.Literal[0, 1] | None],
})


def dyn_cargs(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "antsAI": ants_ai_cargs,
    }.get(t)


def dyn_outputs(
    t: str,
) -> typing.Any:
    """
    Get build outputs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build outputs function.
    """
    return {
        "antsAI": ants_ai_outputs,
    }.get(t)


class AntsAiOutputs(typing.NamedTuple):
    """
    Output object returned when calling `ants_ai(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_transform: OutputPathType
    """The output transform (ITK .mat file)."""


def ants_ai_params(
    metric: typing.Literal["Mattes[fixedImage,movingImage]", "GC[fixedImage,movingImage]", "MI[fixedImage,movingImage]"],
    transform: typing.Literal["Rigid[gradientStep]", "Affine[gradientStep]", "Similarity[gradientStep]", "AlignGeometricCenters", "AlignCentersOfMass"],
    output: str,
    dimensionality: typing.Literal[2, 3] | None = None,
    align_principal_axes: typing.Literal[0, 1] | None = None,
    align_blobs: typing.Literal["numberOfBlobsToExtract", "[numberOfBlobsToExtract,numberOfBlobsToMatch]"] | None = None,
    search_factor: typing.Literal["searchFactor", "[searchFactor,arcFraction]"] | None = None,
    translation_search_grid: typing.Literal["[stepSize, AxBxC]"] | None = None,
    convergence: typing.Literal["numberOfIterations", "[numberOfIterations,convergenceThreshold,convergenceWindowSize]"] | None = None,
    masks: typing.Literal["fixedImageMask", "[fixedImageMask,movingImageMask]"] | None = None,
    random_seed: int | None = None,
    verbose: typing.Literal[0, 1] | None = None,
) -> AntsAiParameters:
    """
    Build parameters.
    
    Args:
        metric: These image metrics are available: Mattes: Mattes mutual\
            information (recommended), GC: global correlation, MI: joint histogram\
            mutual information.
        transform: Several transform options are available. For the rigid,\
            affine, and similarity transforms, the gradientStep characterizes the\
            gradient descent optimization. The other two transform types finds the\
            simple translation transform which aligns the specified image feature.
        output: Specify the output transform (output format an ITK .mat file).
        dimensionality: This option forces the image to be treated as a\
            specified-dimensional image. If not specified, we try to infer the\
            dimensionality from the input image.
        align_principal_axes: Boolean indicating alignment by principal axes.\
            Alternatively, one can align using blobs.
        align_blobs: Boolean indicating alignment by a set of blobs.
        search_factor: Incremental search factor (in degrees) which will sample\
            the arc fraction around the principal axis or default axis.
        translation_search_grid: Translation search grid in mm, which will\
            translate the moving image in each dimension in increments of the step\
            size.
        convergence: Number of iterations.
        masks: Image masks to limit voxels considered by the metric.
        random_seed: Use a fixed seed for random number generation.
        verbose: Verbose output.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "antsAI",
        "metric": metric,
        "transform": transform,
        "output": output,
    }
    if dimensionality is not None:
        params["dimensionality"] = dimensionality
    if align_principal_axes is not None:
        params["align_principal_axes"] = align_principal_axes
    if align_blobs is not None:
        params["align_blobs"] = align_blobs
    if search_factor is not None:
        params["search_factor"] = search_factor
    if translation_search_grid is not None:
        params["translation_search_grid"] = translation_search_grid
    if convergence is not None:
        params["convergence"] = convergence
    if masks is not None:
        params["masks"] = masks
    if random_seed is not None:
        params["random_seed"] = random_seed
    if verbose is not None:
        params["verbose"] = verbose
    return params


def ants_ai_cargs(
    params: AntsAiParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("antsAI")
    if params.get("dimensionality") is not None:
        cargs.extend([
            "--dimensionality",
            str(params.get("dimensionality"))
        ])
    cargs.extend([
        "-m",
        params.get("metric")
    ])
    cargs.extend([
        "-t",
        params.get("transform")
    ])
    if params.get("align_principal_axes") is not None:
        cargs.extend([
            "-p",
            str(params.get("align_principal_axes"))
        ])
    if params.get("align_blobs") is not None:
        cargs.extend([
            "-b",
            params.get("align_blobs")
        ])
    if params.get("search_factor") is not None:
        cargs.extend([
            "-s",
            params.get("search_factor")
        ])
    if params.get("translation_search_grid") is not None:
        cargs.extend([
            "-g",
            params.get("translation_search_grid")
        ])
    if params.get("convergence") is not None:
        cargs.extend([
            "-c",
            params.get("convergence")
        ])
    if params.get("masks") is not None:
        cargs.extend([
            "-x",
            params.get("masks")
        ])
    cargs.extend([
        "-o",
        params.get("output")
    ])
    if params.get("random_seed") is not None:
        cargs.extend([
            "--random-seed",
            str(params.get("random_seed"))
        ])
    if params.get("verbose") is not None:
        cargs.extend([
            "-v",
            str(params.get("verbose"))
        ])
    return cargs


def ants_ai_outputs(
    params: AntsAiParameters,
    execution: Execution,
) -> AntsAiOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AntsAiOutputs(
        root=execution.output_file("."),
        output_transform=execution.output_file(params.get("output") + ".mat"),
    )
    return ret


def ants_ai_execute(
    params: AntsAiParameters,
    execution: Execution,
) -> AntsAiOutputs:
    """
    Program to calculate the optimal linear transform parameters for aligning two
    images.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        execution: The execution object.
    Returns:
        NamedTuple of outputs (described in `AntsAiOutputs`).
    """
    params = execution.params(params)
    cargs = ants_ai_cargs(params, execution)
    ret = ants_ai_outputs(params, execution)
    execution.run(cargs)
    return ret


def ants_ai(
    metric: typing.Literal["Mattes[fixedImage,movingImage]", "GC[fixedImage,movingImage]", "MI[fixedImage,movingImage]"],
    transform: typing.Literal["Rigid[gradientStep]", "Affine[gradientStep]", "Similarity[gradientStep]", "AlignGeometricCenters", "AlignCentersOfMass"],
    output: str,
    dimensionality: typing.Literal[2, 3] | None = None,
    align_principal_axes: typing.Literal[0, 1] | None = None,
    align_blobs: typing.Literal["numberOfBlobsToExtract", "[numberOfBlobsToExtract,numberOfBlobsToMatch]"] | None = None,
    search_factor: typing.Literal["searchFactor", "[searchFactor,arcFraction]"] | None = None,
    translation_search_grid: typing.Literal["[stepSize, AxBxC]"] | None = None,
    convergence: typing.Literal["numberOfIterations", "[numberOfIterations,convergenceThreshold,convergenceWindowSize]"] | None = None,
    masks: typing.Literal["fixedImageMask", "[fixedImageMask,movingImageMask]"] | None = None,
    random_seed: int | None = None,
    verbose: typing.Literal[0, 1] | None = None,
    runner: Runner | None = None,
) -> AntsAiOutputs:
    """
    Program to calculate the optimal linear transform parameters for aligning two
    images.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        metric: These image metrics are available: Mattes: Mattes mutual\
            information (recommended), GC: global correlation, MI: joint histogram\
            mutual information.
        transform: Several transform options are available. For the rigid,\
            affine, and similarity transforms, the gradientStep characterizes the\
            gradient descent optimization. The other two transform types finds the\
            simple translation transform which aligns the specified image feature.
        output: Specify the output transform (output format an ITK .mat file).
        dimensionality: This option forces the image to be treated as a\
            specified-dimensional image. If not specified, we try to infer the\
            dimensionality from the input image.
        align_principal_axes: Boolean indicating alignment by principal axes.\
            Alternatively, one can align using blobs.
        align_blobs: Boolean indicating alignment by a set of blobs.
        search_factor: Incremental search factor (in degrees) which will sample\
            the arc fraction around the principal axis or default axis.
        translation_search_grid: Translation search grid in mm, which will\
            translate the moving image in each dimension in increments of the step\
            size.
        convergence: Number of iterations.
        masks: Image masks to limit voxels considered by the metric.
        random_seed: Use a fixed seed for random number generation.
        verbose: Verbose output.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsAiOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTS_AI_METADATA)
    params = ants_ai_params(
        dimensionality=dimensionality,
        metric=metric,
        transform=transform,
        align_principal_axes=align_principal_axes,
        align_blobs=align_blobs,
        search_factor=search_factor,
        translation_search_grid=translation_search_grid,
        convergence=convergence,
        masks=masks,
        output=output,
        random_seed=random_seed,
        verbose=verbose,
    )
    return ants_ai_execute(params, execution)


__all__ = [
    "ANTS_AI_METADATA",
    "AntsAiOutputs",
    "AntsAiParameters",
    "ants_ai",
    "ants_ai_params",
]
